@@ -30,7 +30,6 @@ Board::Board(KDiamond::GameState* state, KGameDifficulty::standardLevel difficul
 	: QGraphicsScene()
 	, m_selection1(new Diamond(0, 0, 0, 0, KDiamond::Selection, this))
 	, m_selection2(new Diamond(0, 0, 0, 0, KDiamond::Selection, this))
-	, m_background(new QGraphicsPixmapItem(0, this))
 	, m_gameState(state)
 	, m_messenger(new KGamePopupItem)
 	, m_animator(0)
@@ -102,15 +101,12 @@ Board::Board(KDiamond::GameState* state, KGameDifficulty::standardLevel difficul
 				break;
 			}
 			//set diamond
-			m_diamonds[x][y] = new Diamond(x, y, x, y, KDiamond::colorFromNumber(color), this);
+			m_diamonds[x][y] = new Diamond(x, y, x, y, (KDiamond::Color) color, this);
 		}
 	}
 	//init selection markers
 	m_selection1->hide();
 	m_selection2->hide();
-	//init background
-	m_background->setZValue(1);
-	m_background->setAcceptedMouseButtons(0);
 	//init messenger
 	m_messenger->setMessageOpacity(0.8);
 	m_messenger->setHideOnMouseClick(false);
@@ -134,7 +130,6 @@ Board::~Board()
 	delete[] m_diamonds;
 	delete m_selection1;
 	delete m_selection2;
-	delete m_background;
 	delete m_messenger;
 	delete m_animator;
 }
@@ -241,12 +236,7 @@ void Board::resizeScene(int newWidth, int newHeight, bool force)
 	//diamonds
 	emit boardResized(); //give diamonds the chance to change their metrics
 	//background
-	m_background->setPixmap(Renderer::self()->background());
-	QRectF bgRect = m_background->sceneBoundingRect();
-	//The 10.0 and -5.0 are part of a nasty hack to avoid white borders around the background.
-	m_background->scale((newWidth + 10.0) / bgRect.width(), (newHeight + 10.0) / bgRect.height());
-	m_background->setPos(-5.0, -5.0);
-	m_background->setVisible(true);
+	setBackgroundBrush(Renderer::self()->background());
 }
 
 void Board::mouseOnDiamond(int xIndex, int yIndex)
@@ -381,7 +371,7 @@ void Board::timerEvent(QTimerEvent* event)
 			{
 				//no diamond rows were formed by the last move -> revoke movement (unless we are in a cascade)
 				if (m_swapping1x != -1 && m_swapping1y != -1 && m_swapping2x != -1 && m_swapping2y != -1)
-					m_jobQueue << KDiamond::RevokeSwapDiamondsJob;
+					m_jobQueue.prepend(KDiamond::RevokeSwapDiamondsJob);
 				else
 					m_jobQueue << KDiamond::UpdateAvailableMovesJob;
 			}
@@ -575,7 +565,7 @@ void Board::fillGaps()
 			if (m_diamonds[x][y] != 0)
 				continue; //inside of diamond stack - no gaps to fill
 			--yt;
-			m_diamonds[x][y] = new Diamond(x, y, x, yt, KDiamond::colorFromNumber(qrand() % m_colorCount + 1), this);
+			m_diamonds[x][y] = new Diamond(x, y, x, yt, (KDiamond::Color) (qrand() % m_colorCount + 1), this);
 			m_diamonds[x][y]->setPosInBoardCoords(QPointF(x, yt));
 			m_diamonds[x][y]->updateGeometry();
 			m_animator->addItem(m_diamonds[x][y], QPointF(x, yt), QPointF(x, y));
