@@ -1,31 +1,29 @@
 /*
- *  Killbots
- *  Copyright (C) 2006-2008  Parker Coates <parker.coates@gmail.com>
+ *  Copyright 2006-2009  Parker Coates <parker.coates@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License as
- *  published by the Free Software Foundation; either version 2 of
- *  the License, or (at your option) any later version.
+ *  This file is part of Killbots.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Killbots is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Killbots is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses>
- *  or write to the Free Software Foundation, Inc., 51 Franklin Street,
- *  Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with Killbots. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "engine.h"
 
-#include "scene.h"
+#include "coordinator.h"
 #include "settings.h"
 #include "sprite.h"
 
 #include <KDE/KDebug>
-#include <KDE/KLocale>
 #include <KDE/KRandom>
 
 uint qHash( const QPoint & point )
@@ -40,16 +38,10 @@ inline int sign( int num )
 }
 
 
-Killbots::Engine::Engine( Scene * scene, QObject * parent )
+Killbots::Engine::Engine( Killbots::Coordinator* scene, QObject* parent )
   : QObject( parent ),
-    m_scene( scene ),
+    m_coordinator( scene ),
     m_hero( 0 ),
-    m_busyAnimating( false ),
-    m_processFastbots( false ),
-    m_gameOver( false ),
-    m_newGameRequested( false ),
-    m_repeatedAction( NoAction ),
-    m_queuedAction( NoAction ),
     m_rules( 0 ),
     m_round( 0 ),
     m_score( 0 ),
@@ -58,9 +50,10 @@ Killbots::Engine::Engine( Scene * scene, QObject * parent )
     m_robotCount( 0.0 ),
     m_fastbotCount( 0.0 ),
     m_junkheapCount( 0.0 ),
+    m_heroIsDead( false ),
+    m_waitingOutRound( false ),
     m_spriteMap()
 {
-	connect( m_scene, SIGNAL(animationDone()), this, SLOT(animationDone()), Qt::QueuedConnection );
 }
 
 
@@ -69,347 +62,220 @@ Killbots::Engine::~Engine()
 	delete m_rules;
 }
 
+void Killbots::Engine::setRuleset( const Ruleset * ruleset )
+{
+	if ( ruleset && ruleset != m_rules )
+	{
+		delete m_rules;
+		m_rules = ruleset;
+	}
+}
 
-const Killbots::Ruleset * Killbots::Engine::ruleset()
+
+const Killbots::Ruleset * Killbots::Engine::ruleset() const
 {
 	return m_rules;
 }
 
 
-bool Killbots::Engine::gameHasStarted()
+bool Killbots::Engine::gameHasStarted() const
 {
 	return m_hero && m_score > 0;
 }
 
 
-void Killbots::Engine::requestNewGame()
+bool Killbots::Engine::isRoundComplete() const
 {
-	if ( !m_busyAnimating || m_gameOver )
-		newGame();
-	else
-		m_newGameRequested = true;
+	return m_bots.isEmpty();
 }
 
 
-void Killbots::Engine::newGame()
+bool Killbots::Engine::isHeroDead() const
 {
-	if ( !m_rules || m_rules->fileName() != Settings::ruleset() )
-	{
-		delete m_rules;
-		m_rules = 0;
-		m_rules = Ruleset::load( Settings::ruleset() );
-	}
-	if ( !m_rules )
-		m_rules = Ruleset::loadDefault();
-	Q_ASSERT( m_rules != 0 );
+	return m_heroIsDead;
+}
 
-	bool energyIncluded = m_rules->energyAtGameStart() > 0 || m_rules->maxEnergyAtGameStart() > 0 || m_rules->maxEnergyAddedEachRound() > 0;
-	emit newGame( m_rules->rows(), m_rules->columns(), energyIncluded );
+
+bool Killbots::Engine::isBoardFull() const
+{
+	return m_robotCount + m_fastbotCount + m_junkheapCount
+	       > m_rules->rows() * m_rules->columns() / 2;
+}
+
+
+bool Killbots::Engine::canSafeTeleport() const
+{
+	return m_rules->safeTeleportEnabled()
+	       && m_energy >= m_rules->costOfSafeTeleport();
+}
+
+
+bool Killbots::Engine::canUseVaporizer() const
+{
+	return m_rules->vaporizerEnabled()
+	       && m_energy >= m_rules->costOfVaporizer();
+}
+
+
+void Killbots::Engine::startNewGame()
+{
+	Q_ASSERT( m_rules != 0 );
 
 	// Don't show the new game message on first start.
 	if ( m_round != 0 )
-	{
-		m_scene->showNewGameMessage();
-	}
+		 emit showNewGameMessage();
 
-	m_newGameRequested = false;
-	m_gameOver = false;
-	m_score = 0;
+	m_heroIsDead = false;
 
 	m_round = 1;
-
-	m_maxEnergy = m_rules->maxEnergyAtGameStart();
-	m_energy = m_rules->energyAtGameStart();
+	m_score = 0;
+	m_maxEnergy = m_rules->energyEnabled() ? m_rules->maxEnergyAtGameStart() : 0;
+	m_energy = m_rules->energyEnabled() ? m_rules->energyAtGameStart() : 0;
 	m_robotCount = m_rules->enemiesAtGameStart();
 	m_fastbotCount = m_rules->fastEnemiesAtGameStart();
 	m_junkheapCount = m_rules->junkheapsAtGameStart();
 
 	emit teleportAllowed( true );
-	emit teleportSafelyAllowed( m_energy >= m_rules->costOfSafeTeleport() );
 	emit waitOutRoundAllowed( true );
+	emit teleportSafelyAllowed( canSafeTeleport() );
+	emit vaporizerAllowed( canUseVaporizer() );
 
-	newRound(false);
-	m_busyAnimating = true;
-	m_scene->startAnimation();
+	// Code used to generate theme previews
+	//newRound( "  r\nhjf", false );
+
+	startNewRound( false );
 }
 
 
-void Killbots::Engine::requestAction( HeroAction action )
-{
-	// If the hero is doing a repeated move the new request only stops the hero.
-	bool cancelRepeatedMove = m_repeatedAction != WaitOutRound && m_repeatedAction != NoAction;
-	if ( cancelRepeatedMove )
-	{
-		m_repeatedAction = NoAction;
-	}
-
-	if ( !m_busyAnimating && !m_gameOver )
-	{
-		doAction( action );
-	}
-	else if ( !cancelRepeatedMove )
-	{
-		m_queuedAction = action;
-	}
-}
-
-
-// This slot is provided only for QSignalMapper compatibility.
-void Killbots::Engine::requestAction( int action )
-{
-	requestAction( static_cast<HeroAction>( action ) );
-}
-
-
-void Killbots::Engine::doAction( HeroAction action )
-{
-	bool actionSuccessful = false;
-	bool boardFull = false;
-
-	refreshSpriteMap();
-
-	if ( action <= Hold )
-	{
-		actionSuccessful = moveHero( action );
-	}
-	else if ( ( action == TeleportSafely || action == TeleportSafelyIfPossible ) && m_energy >= m_rules->costOfSafeTeleport() )
-	{
-		actionSuccessful = teleportHeroSafely();
-		boardFull = !actionSuccessful;
-	}
-	else if ( action == Teleport || action == TeleportSafelyIfPossible )
-	{
-		actionSuccessful = teleportHero();
-	}
-	else if ( action == WaitOutRound )
-	{
-		m_repeatedAction = WaitOutRound;
-		actionSuccessful = true;
-	}
-
-	if ( actionSuccessful )
-	{
-		moveRobots();
-		assessDamage();
-		if ( m_bots.isEmpty() )
-			m_scene->showRoundCompleteMessage();
-		else
-			m_processFastbots = true;
-		m_busyAnimating = true;
-		m_scene->startAnimation();
-	}
-	else if ( boardFull )
-	{
-		resetBotCounts();
-		m_busyAnimating = true;
-		m_scene->startAnimation();
-	}
-}
-
-
-void Killbots::Engine::animationDone()
-{
-	m_busyAnimating = false;
-
-	if ( m_newGameRequested )
-	{
-		newGame();
-	}
-	else if ( m_processFastbots )
-	{
-		m_processFastbots = false;
-		moveRobots( true );
-		assessDamage();
-		if ( m_bots.isEmpty() )
-			m_scene->showRoundCompleteMessage();
-		m_busyAnimating = true;
-		m_scene->startAnimation();
-	}
-	else if ( m_gameOver )
-	{
-		m_scene->showGameOverMessage();
-		emit teleportAllowed( false );
-		emit teleportSafelyAllowed( false );
-		emit waitOutRoundAllowed( false );
-		emit gameOver( m_score, m_round );
-	}
-	else if ( m_bots.isEmpty() )
-	{
-		newRound();
-		if ( m_robotCount + m_fastbotCount + m_junkheapCount > m_rules->rows() * m_rules->columns() / 2 )
-			resetBotCounts();
-		m_busyAnimating = true;
-		m_scene->startAnimation();
-	}
-	else if ( m_repeatedAction != NoAction )
-	{
-		doAction( m_repeatedAction );
-	}
-	else if ( m_queuedAction != NoAction )
-	{
-		doAction( m_queuedAction );
-		m_queuedAction = NoAction;
-	}
-}
-
-
-void Killbots::Engine::newRound( bool incrementRound )
+void Killbots::Engine::startNewRound( bool incrementRound, const QString & layout )
 {
 	cleanUpRound();
 
-	m_processFastbots = false;
-	m_repeatedAction = NoAction;
-	m_queuedAction = NoAction;
+	m_waitingOutRound = false;
 
-	m_scene->beginNewAnimationStage();
+	m_coordinator->beginNewAnimationStage();
 
 	if ( incrementRound )
 	{
 		++m_round;
 
-		m_maxEnergy += m_rules->maxEnergyAddedEachRound();
-		if ( m_energy < m_maxEnergy )
-			m_energy = qMin( m_energy + m_rules->energyAddedEachRound(), int( m_maxEnergy ) );
+		if ( m_rules->energyEnabled() )
+		{
+			m_maxEnergy += m_rules->maxEnergyAddedEachRound();
+			updateEnergy( m_rules->energyAddedEachRound() );
+		}
 		m_robotCount += m_rules->enemiesAddedEachRound();
 		m_fastbotCount += m_rules->fastEnemiesAddedEachRound();
 		m_junkheapCount += m_rules->junkheapsAddedEachRound();
 	}
 
-	// Place the hero in the centre of the board.
-	QPoint centre = QPoint( qRound( m_rules->columns() / 2 ), qRound( m_rules->rows() / 2 ) );
-	m_hero = m_scene->createSprite( Hero, centre );
-	m_spriteMap.insert( centre, m_hero );
-
-	// Create and randomly place junkheaps.
-	for ( int i = m_junkheapCount; i > 0 ; --i )
+	if ( layout.isEmpty() )
 	{
-		QPoint point;
-		do
-			point = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
-		while ( m_spriteMap.contains( point ) );
+		// Place the hero in the centre of the board.
+		const QPoint centre = QPoint( qRound( m_rules->columns() / 2 ), qRound( m_rules->rows() / 2 ) );
+		m_hero = m_coordinator->createSprite( Hero, centre );
 
-		m_junkheaps << m_scene->createSprite( Junkheap, point );
-		m_spriteMap.insert( point, m_junkheaps.last() );
+		// Create and randomly place junkheaps.
+		for ( int i = m_junkheapCount; i > 0 ; --i )
+		{
+			const QPoint point = randomEmptyCell();
+			m_junkheaps << m_coordinator->createSprite( Junkheap, point );
+			m_spriteMap.insert( point, m_junkheaps.last() );
+		}
+
+		// Create and randomly place robots.
+		for ( int i = m_robotCount; i > 0; --i )
+		{
+			const QPoint point = randomEmptyCell();
+			m_bots << m_coordinator->createSprite( Robot, point );
+			m_spriteMap.insert( point, m_bots.last() );
+		}
+
+		// Create and randomly place fastbots.
+		for ( int i = m_fastbotCount; i > 0; --i )
+		{
+			const QPoint point = randomEmptyCell();
+			m_bots << m_coordinator->createSprite( Fastbot, point );
+			m_spriteMap.insert( point, m_bots.last() );
+		}
+	}
+	else
+	{
+		const QStringList rows = layout.split('\n');
+		for ( int r = 0; r < rows.size(); ++r )
+		{
+			for ( int c = 0; c < rows.at( r ).size(); ++c )
+			{
+				const QChar ch = rows.at( r ).at( c );
+				const QPoint point( c, r );
+
+				if ( ch == 'h' && m_hero == 0 )
+					m_hero = m_coordinator->createSprite( Hero, point );
+				else if ( ch == 'r' )
+					m_bots << m_coordinator->createSprite( Robot, point );
+				else if ( ch == 'f' )
+					m_bots << m_coordinator->createSprite( Fastbot, point );
+				else if ( ch == 'j' )
+					m_junkheaps << m_coordinator->createSprite( Junkheap, point );
+			}
+		}
 	}
 
-	// Create and randomly place robots.
-	for ( int i = m_robotCount; i > 0; --i )
-	{
-		QPoint point;
-		do
-			point = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
-		while ( m_spriteMap.contains( point ) );
-
-		m_bots << m_scene->createSprite( Robot, point );
-		m_spriteMap.insert( point, m_bots.last() );
-	}
-
-	// Create and randomly place fastbots.
-	for ( int i = m_fastbotCount; i > 0; --i )
-	{
-		QPoint point;
-		do
-			point = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
-		while ( m_spriteMap.contains( point ) );
-
-		m_bots << m_scene->createSprite( Fastbot, point );
-		m_spriteMap.insert( point, m_bots.last() );
-	}
-
-	// Code used to generate theme previews
-// 	m_hero = m_scene->createSprite( Hero, QPoint( 0, 1 ) );
-// 	m_junkheaps << m_scene->createSprite( Junkheap, QPoint( 1, 1 ) );
-// 	m_bots << m_scene->createSprite( Robot, QPoint( 2, 0 ) );
-// 	m_bots << m_scene->createSprite( Fastbot, QPoint( 2, 1 ) );
+	emit roundChanged( m_round );
+	emit scoreChanged( m_score );
+	emit enemyCountChanged( m_bots.size() );
+	emit energyChanged( m_energy );
 
 	refreshSpriteMap();
-
-	m_scene->updateRound( m_round );
-	m_scene->updateScore( m_score );
-	m_scene->updateEnemyCount( m_bots.size() );
-	m_scene->updateEnergy( m_energy );
-	emit teleportSafelyAllowed( m_energy >= m_rules->costOfSafeTeleport() );
 }
 
 
-bool Killbots::Engine::moveHero( HeroAction direction )
+// Returns true if the move was performed, returns false otherwise.
+bool Killbots::Engine::moveHero( Killbots::HeroAction direction )
 {
-	QPoint newCell = m_hero->gridPos() + offsetFromDirection( direction );
+	refreshSpriteMap();
+	const QPoint newCell = m_hero->gridPos() + offsetFromDirection( direction );
+	const bool preventUnsafeMoves = Settings::preventUnsafeMoves() || direction < 0;
 
-	if ( moveIsValid( newCell, direction )
-	     && ( moveIsSafe( newCell, direction )
-	          || ( !Settings::preventUnsafeMoves()
-	               && m_repeatedAction == NoAction
-	             )
-	        )
-	   )
+	if ( moveIsValid( newCell, direction ) && ( moveIsSafe( newCell, direction ) || !preventUnsafeMoves ) )
 	{
-		if ( direction < 0 )
-			m_repeatedAction = direction;
-		else
-			m_repeatedAction = NoAction;
-
 		if ( direction != Hold )
 		{
-			m_scene->beginNewAnimationStage();
+			m_coordinator->beginNewAnimationStage();
 
 			if ( spriteTypeAt( newCell ) == Junkheap )
 				pushJunkheap( m_spriteMap.value( newCell ), direction );
 
-			m_scene->slideSprite( m_hero, newCell );
+			m_coordinator->slideSprite( m_hero, newCell );
 		}
 		return true;
 	}
 	else
 	{
-		m_repeatedAction = NoAction;
 		return false;
 	}
 }
 
 
-void Killbots::Engine::pushJunkheap( Sprite * junkheap, HeroAction direction )
-{
-	QPoint nextCell = junkheap->gridPos() + offsetFromDirection( direction );
-
-	Sprite * currentOccupant = m_spriteMap.value( nextCell );
-	if ( currentOccupant )
-	{
-		if ( currentOccupant->spriteType() == Junkheap )
-			pushJunkheap( currentOccupant, direction );
-		else
-		{
-			destroySprite( currentOccupant );
-			m_score += m_rules->squashKillPointBonus();
-			if ( m_energy < m_maxEnergy )
-				m_energy = qMin( m_energy + m_rules->squashKillEnergyBonus(), int( m_maxEnergy ) );
-			m_scene->updateEnergy( m_energy );
-			emit teleportSafelyAllowed( m_energy >= m_rules->costOfSafeTeleport() );
-		}
-	}
-
-	m_scene->slideSprite( junkheap, nextCell );
-}
-
-
+// Always returns true as teleports always succeed.
 bool Killbots::Engine::teleportHero()
 {
-	QPoint point;
-	do
-		point = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
-	while ( spriteTypeAt( point ) != NoSprite || point == m_hero->gridPos() );
-
-	m_scene->beginNewAnimationStage();
-	m_scene->teleportSprite( m_hero, point );
-
+	refreshSpriteMap();
+	const QPoint point = randomEmptyCell();
+	m_coordinator->beginNewAnimationStage();
+	m_coordinator->teleportSprite( m_hero, point );
 	return true;
 }
 
 
+// Returns true if a safe cell was found. If no safe cell was found than
+// the board must be full.
 bool Killbots::Engine::teleportHeroSafely()
 {
+	refreshSpriteMap();
+
 	// Choose a random cell...
-	QPoint startPoint = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
+	const QPoint startPoint = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
 	QPoint point = startPoint;
 
 	// ...and step through all the cells on the board looking for a safe cell.
@@ -439,28 +305,59 @@ bool Killbots::Engine::teleportHeroSafely()
 	}
 	else
 	{
-		m_scene->beginNewAnimationStage();
-
-		m_scene->updateEnergy( m_energy -= m_rules->costOfSafeTeleport() );
-		emit teleportSafelyAllowed( m_energy >= m_rules->costOfSafeTeleport() );
-
-		m_scene->teleportSprite( m_hero, point );
+		m_coordinator->beginNewAnimationStage();
+		updateEnergy( -m_rules->costOfSafeTeleport() );
+		m_coordinator->teleportSprite( m_hero, point );
 
 		return true;
 	}
 }
 
 
+// Returns true if any enemies were within range.
+bool Killbots::Engine::useVaporizer()
+{
+	refreshSpriteMap();
+	QList<Sprite *> neighbors;
+	for ( int i = Right; i <= DownRight; ++i )
+	{
+		const QPoint neighbor = m_hero->gridPos() + offsetFromDirection( i );
+		if ( cellIsValid( neighbor ) && ( spriteTypeAt( neighbor ) == Robot || spriteTypeAt( neighbor ) == Fastbot ) )
+			neighbors << m_spriteMap.value(neighbor);
+	}
+
+	if ( !neighbors.isEmpty() )
+	{
+		m_coordinator->beginNewAnimationStage();
+		foreach ( Sprite * sprite, neighbors )
+			destroySprite( sprite );
+		updateEnergy( -m_rules->costOfVaporizer() );
+		return true;
+	}
+	else
+	{
+		return false;
+	}
+}
+
+
+bool Killbots::Engine::waitOutRound()
+{
+	m_waitingOutRound = true;
+	return true;
+}
+
+
 void Killbots::Engine::moveRobots( bool justFastbots )
 {
-	m_scene->beginNewAnimationStage();
+	m_coordinator->beginNewAnimationStage();
 
 	foreach( Sprite * bot, m_bots )
 	{
 		if ( bot->spriteType() == Fastbot || !justFastbots )
 		{
-			QPoint offset( sign( m_hero->gridPos().x() - bot->gridPos().x() ), sign( m_hero->gridPos().y() - bot->gridPos().y() ) );
-			m_scene->slideSprite( bot, bot->gridPos() + offset );
+			const QPoint offset( sign( m_hero->gridPos().x() - bot->gridPos().x() ), sign( m_hero->gridPos().y() - bot->gridPos().y() ) );
+			m_coordinator->slideSprite( bot, bot->gridPos() + offset );
 		}
 	}
 }
@@ -470,24 +367,24 @@ void Killbots::Engine::assessDamage()
 {
 	refreshSpriteMap();
 
-	m_scene->beginNewAnimationStage();
+	m_coordinator->beginNewAnimationStage();
 
 	if ( m_spriteMap.count( m_hero->gridPos() ) > 0 )
-		m_gameOver = true;
+		m_heroIsDead = true;
 
 	// Check junkheaps for dead robots
 	foreach ( Sprite * junkheap, m_junkheaps )
-		destroyAllCollidingBots( junkheap, !m_gameOver );
+		destroyAllCollidingBots( junkheap, !m_heroIsDead );
 
 	// Check for robot-on-robot violence
 	int i = 0;
 	while ( i < m_bots.size() )
 	{
 		Sprite * bot = m_bots[i];
-		if ( bot->gridPos() != m_hero->gridPos() && destroyAllCollidingBots( bot, !m_gameOver ) )
+		if ( bot->gridPos() != m_hero->gridPos() && destroyAllCollidingBots( bot, !m_heroIsDead ) )
 		{
-			m_junkheaps << m_scene->createSprite( Junkheap, bot->gridPos() );
-			destroySprite( bot, !m_gameOver );
+			m_junkheaps << m_coordinator->createSprite( Junkheap, bot->gridPos() );
+			destroySprite( bot, !m_heroIsDead );
 		}
 		else
 		{
@@ -495,96 +392,43 @@ void Killbots::Engine::assessDamage()
 		}
 	}
 
-	m_scene->updateScore( m_score );
-	m_scene->updateEnemyCount( m_bots.size() );
-	m_scene->updateEnergy( m_energy );
-	emit teleportSafelyAllowed( m_energy >= m_rules->costOfSafeTeleport() );
-}
-
-
-void Killbots::Engine::cleanUpRound()
-{
-	m_scene->beginNewAnimationStage();
-
-	if ( m_hero )
-		destroySprite( m_hero );
-	m_hero = 0;
-
-	foreach( Sprite * bot, m_bots )
-		destroySprite( bot, false );
-	m_bots.clear();
-
-	foreach( Sprite * junkheap, m_junkheaps )
-		destroySprite( junkheap );
-	m_junkheaps.clear();
-
-	m_spriteMap.clear();
-}
-
-
-void Killbots::Engine::destroySprite( Sprite * sprite, bool calculatePoints )
-{
-	SpriteType type = sprite->spriteType();
-
-	if ( type == Robot || type == Fastbot)
+	if ( isRoundComplete() )
 	{
-		if ( calculatePoints )
-		{
-			if ( type == Robot )
-				m_score += m_rules->pointsPerEnemyKilled();
-			else
-				m_score += m_rules->pointsPerFastEnemyKilled();
-
-			if ( m_repeatedAction == WaitOutRound )
-			{
-				m_score += m_rules->waitKillPointBonus();
-				if ( m_energy < m_maxEnergy )
-					m_energy = qMin( m_energy + m_rules->waitKillEnergyBonus(), int( m_maxEnergy ) );
-			}
-		}
-		m_bots.removeOne( sprite );
+		m_coordinator->beginNewAnimationStage();
+		emit showRoundCompleteMessage();
 	}
-	else if ( type == Junkheap )
-	{
-		m_junkheaps.removeOne( sprite );
-	}
-
-	m_scene->destroySprite( sprite );
-}
-
-
-bool Killbots::Engine::destroyAllCollidingBots( const Sprite * sprite, bool calculatePoints )
-{
-	bool result = false;
-
-	foreach ( Sprite * robot, m_spriteMap.values( sprite->gridPos() ) )
-	{
-		if ( robot != sprite && ( robot->spriteType() == Robot || robot->spriteType() == Fastbot ) )
-		{
-			destroySprite( robot, calculatePoints );
-			result = true;
-		}
-	}
-
-	return result;
 }
 
 
 void Killbots::Engine::resetBotCounts()
 {
-	m_scene->beginNewAnimationStage();
-	m_scene->showBoardFullMessage();
+	m_coordinator->beginNewAnimationStage();
+	emit showBoardFullMessage();
 
 	m_maxEnergy = m_rules->maxEnergyAtGameStart();
 	m_robotCount = m_rules->enemiesAtGameStart();
 	m_fastbotCount = m_rules->fastEnemiesAtGameStart();
 	m_junkheapCount = m_rules->junkheapsAtGameStart();
 
-	m_scene->beginNewAnimationStage();
-	newRound(false);
+	m_coordinator->beginNewAnimationStage();
+	startNewRound(false);
 }
 
 
+void Killbots::Engine::endGame()
+{
+	emit showGameOverMessage();
+	emit teleportAllowed( false );
+	emit waitOutRoundAllowed( false );
+	emit teleportSafelyAllowed( false );
+	emit vaporizerAllowed( false );
+	emit gameOver( m_score, m_round );
+}
+
+
+// The hero action functions and the assessDamage functions must know the
+// contents of each cell. This function updates the hash that maps cells to
+// their contents.
 void Killbots::Engine::refreshSpriteMap()
 {
 	m_spriteMap.clear();
@@ -595,16 +439,57 @@ void Killbots::Engine::refreshSpriteMap()
 }
 
 
+// A convenience function to query the type of a sprite any the given cell.
 int Killbots::Engine::spriteTypeAt( const QPoint & cell ) const
 {
-	Sprite * occupant = m_spriteMap.value( cell );
-	if ( occupant )
-		return occupant->spriteType();
+	if ( m_spriteMap.contains( cell ) )
+		return m_spriteMap.value( cell )->spriteType();
 	else
 		return NoSprite;
 }
 
 
+QPoint Killbots::Engine::offsetFromDirection( int direction ) const
+{
+	if ( direction < 0 )
+		direction = -direction - 1;
+
+	switch( direction )
+	{
+	case Right:
+		return QPoint(  1,  0 );
+	case UpRight:
+		return QPoint(  1, -1 );
+	case Up:
+		return QPoint(  0, -1 );
+	case UpLeft:
+		return QPoint( -1, -1 );
+	case Left:
+		return QPoint( -1,  0 );
+	case DownLeft:
+		return QPoint( -1,  1 );
+	case Down:
+		return QPoint(  0,  1 );
+	case DownRight:
+		return QPoint(  1,  1 );
+	default:
+		return QPoint(  0,  0 );
+	};
+}
+
+
+// Returns a random empty cell on the grid. Depends on a fresh spritemap.
+QPoint Killbots::Engine::randomEmptyCell() const
+{
+	QPoint point;
+	do
+		point = QPoint( KRandom::random() % m_rules->columns(), KRandom::random() % m_rules->rows() );
+	while ( spriteTypeAt( point ) != NoSprite || point == m_hero->gridPos() );
+	return point;
+}
+
+
+// Returns true if the given cell lies inside the game grid.
 bool Killbots::Engine::cellIsValid( const QPoint & cell ) const
 {
 	return ( 0 <= cell.x()
@@ -617,19 +502,19 @@ bool Killbots::Engine::cellIsValid( const QPoint & cell ) const
 
 bool Killbots::Engine::moveIsValid( const QPoint & cell, HeroAction direction ) const
 {
-	QPoint cellBehindCell = cell + offsetFromDirection( direction );
+	const QPoint cellBehindCell = cell + offsetFromDirection( direction );
 
-/*	// The short version
+	// The short version
 	return ( cellIsValid( cell )
-	         && ( spriteTypeAt( cell ) == ""
-	              || ( spriteTypeAt( cell ) == "junkheap"
+	         && ( spriteTypeAt( cell ) == NoSprite
+	              || ( spriteTypeAt( cell ) == Junkheap
 	                   && canPushJunkheap( m_spriteMap.value( cell ), direction )
 	                 )
 	            )
 	       );
-*/
 
-	// The debuggable version
+
+/*	// The debuggable version
 	bool result = true;
 
 	if ( cellIsValid( cell ) )
@@ -658,6 +543,7 @@ bool Killbots::Engine::moveIsValid( const QPoint & cell, HeroAction direction )
 	}
 
 	return result;
+*/
 }
 
 
@@ -764,12 +650,14 @@ bool Killbots::Engine::moveIsSafe( const QPoint & cell, HeroAction direction ) c
 	bool result = true;
 
 	// If we're pushing a junkheap, store the cell that the junkheap will end up in. Otherwise store an invalid cell.
-	QPoint cellBehindJunkheap = ( spriteTypeAt( cell ) != Junkheap ) ? QPoint( -1, -1 ) : cell + offsetFromDirection( direction );
+	const QPoint cellBehindJunkheap = ( spriteTypeAt( cell ) != Junkheap )
+	                                  ? QPoint( -1, -1 )
+	                                  : cell + offsetFromDirection( direction );
 
 	// We check the each of the target cells neighbours.
 	for ( int i = Right; i <= DownRight && result; ++i )
 	{
-		QPoint neighbor = cell + offsetFromDirection( i );
+		const QPoint neighbor = cell + offsetFromDirection( i );
 
 		// If the neighbour is invalid or the cell behind the junkheap, continue to the next neighbour.
 		if ( !cellIsValid( neighbor ) || spriteTypeAt( neighbor ) == Junkheap || neighbor == cellBehindJunkheap )
@@ -777,11 +665,13 @@ bool Killbots::Engine::moveIsSafe( const QPoint & cell, HeroAction direction ) c
 
 		// If the neighbour contains an enemy, the move is unsafe.
 		if ( spriteTypeAt( neighbor ) == Robot || spriteTypeAt( neighbor ) == Fastbot )
+		{
 			result = false;
+		}
 		else
 		{
 			// neighboursNeighbour is the cell behind the neighbour, with respect to the target cell.
-			QPoint neighborsNeighbor = neighbor + offsetFromDirection( i );
+			const QPoint neighborsNeighbor = neighbor + offsetFromDirection( i );
 
 			// If we're examining a diagonal neighbour (an odd direction)...
 			if ( i % 2 == 1 )
@@ -796,9 +686,11 @@ bool Killbots::Engine::moveIsSafe( const QPoint & cell, HeroAction direction ) c
 				// Assemble a list of the cells behind the neighbour.
 				QList<QPoint> cellsBehindNeighbor;
 				cellsBehindNeighbor << neighborsNeighbor;
-				// Add neighboursNeighbour's anticlockwise neighbour. ( i + 2 ) % 8 is the direction a quarter turn anticlockwise from i.
+				// Add neighboursNeighbour's anticlockwise neighbour.
+				// ( i + 2 ) % 8 is the direction a quarter turn anticlockwise from i.
 				cellsBehindNeighbor << neighborsNeighbor + offsetFromDirection( ( i + 2 ) % 8 );
-				// Add neighboursNeighbour's clockwise neighbour. ( i + 6 ) % 8 is the direction a quarter turn clockwise from i.
+				// Add neighboursNeighbour's clockwise neighbour.
+				// ( i + 6 ) % 8 is the direction a quarter turn clockwise from i.
 				cellsBehindNeighbor << neighborsNeighbor + offsetFromDirection( ( i + 6 ) % 8 );
 
 				// Then we just count the number of fastbots and robots in the list of cells.
@@ -825,7 +717,9 @@ bool Killbots::Engine::moveIsSafe( const QPoint & cell, HeroAction direction ) c
 
 bool Killbots::Engine::canPushJunkheap( const Sprite * junkheap, HeroAction direction ) const
 {
-	QPoint nextCell = junkheap->gridPos() + offsetFromDirection( direction );
+	Q_ASSERT( junkheap->spriteType() == Junkheap );
+
+	const QPoint nextCell = junkheap->gridPos() + offsetFromDirection( direction );
 
 	if ( m_rules->pushableJunkheaps() != Ruleset::None && cellIsValid( nextCell ) )
 	{
@@ -837,34 +731,161 @@ bool Killbots::Engine::canPushJunkheap( const Sprite * junkheap, HeroAction dire
 			return m_rules->squaskKillsEnabled();
 	}
 	else
-		return false;
-}
-
-
-QPoint Killbots::Engine::offsetFromDirection( int direction ) const
-{
-	if ( direction < 0 )
-		direction = -direction - 1;
-
-	switch( direction )
 	{
-	case Right:
-		return QPoint(  1,  0 );
-	case UpRight:
-		return QPoint(  1, -1 );
-	case Up:
-		return QPoint(  0, -1 );
-	case UpLeft:
-		return QPoint( -1, -1 );
-	case Left:
-		return QPoint( -1,  0 );
-	case DownLeft:
-		return QPoint( -1,  1 );
-	case Down:
-		return QPoint(  0,  1 );
-	case DownRight:
-		return QPoint(  1,  1 );
-	default:
-		return QPoint(  0,  0 );
-	};
+		return false;
+	}
+}
+
+
+void Killbots::Engine::pushJunkheap( Sprite * junkheap, HeroAction direction )
+{
+	const QPoint nextCell = junkheap->gridPos() + offsetFromDirection( direction );
+	Sprite * currentOccupant = m_spriteMap.value( nextCell );
+	if ( currentOccupant )
+	{
+		if ( currentOccupant->spriteType() == Junkheap )
+		{
+			pushJunkheap( currentOccupant, direction );
+		}
+		else
+		{
+			destroySprite( currentOccupant );
+			updateScore( m_rules->squashKillPointBonus() );
+			updateEnergy( m_rules->squashKillEnergyBonus() );
+		}
+	}
+
+	m_coordinator->slideSprite( junkheap, nextCell );
+}
+
+
+void Killbots::Engine::cleanUpRound()
+{
+	m_coordinator->beginNewAnimationStage();
+
+	if ( m_hero )
+		destroySprite( m_hero );
+	m_hero = 0;
+
+	foreach( Sprite * bot, m_bots )
+		destroySprite( bot, false );
+	m_bots.clear();
+
+	foreach( Sprite * junkheap, m_junkheaps )
+		destroySprite( junkheap );
+	m_junkheaps.clear();
+
+	m_spriteMap.clear();
+}
+
+
+void Killbots::Engine::destroySprite( Sprite * sprite, bool calculatePoints )
+{
+	const SpriteType type = sprite->spriteType();
+
+	if ( type == Robot || type == Fastbot)
+	{
+		if ( calculatePoints )
+		{
+			if ( type == Robot )
+				updateScore( m_rules->pointsPerEnemyKilled() );
+			else
+				updateScore( m_rules->pointsPerFastEnemyKilled() );
+
+			if ( m_waitingOutRound )
+			{
+				updateScore(  m_rules->waitKillPointBonus() );
+				updateEnergy( m_rules->waitKillEnergyBonus() );
+			}
+		}
+		m_bots.removeOne( sprite );
+		emit enemyCountChanged( m_bots.size() );
+	}
+	else if ( type == Junkheap )
+	{
+		m_junkheaps.removeOne( sprite );
+	}
+
+	m_coordinator->destroySprite( sprite );
+}
+
+
+bool Killbots::Engine::destroyAllCollidingBots( const Sprite * sprite, bool calculatePoints )
+{
+	bool result = false;
+
+	foreach ( Sprite * robot, m_spriteMap.values( sprite->gridPos() ) )
+	{
+		if ( robot != sprite && ( robot->spriteType() == Robot || robot->spriteType() == Fastbot ) )
+		{
+			destroySprite( robot, calculatePoints );
+			result = true;
+		}
+	}
+
+	return result;
+}
+
+
+void Killbots::Engine::updateScore( int changeInScore )
+{
+	if ( changeInScore != 0 )
+	{
+		m_score = m_score + changeInScore;
+		emit scoreChanged( m_score );
+	}
+}
+
+
+void Killbots::Engine::updateEnergy( int changeInEnergy )
+{
+	if ( m_rules->energyEnabled() && changeInEnergy != 0 )
+	{
+		if ( changeInEnergy > 0 && m_energy > int( m_maxEnergy ) )
+		{
+			m_score += changeInEnergy * m_rules->pointsPerEnergyAboveMax();
+		}
+		else if ( changeInEnergy > 0 && m_energy + changeInEnergy > int( m_maxEnergy ) )
+		{
+			m_score += ( m_energy + changeInEnergy - int( m_maxEnergy ) ) * m_rules->pointsPerEnergyAboveMax();
+			m_energy = int( m_maxEnergy );
+		}
+		else
+		{
+			m_energy = m_energy + changeInEnergy;
+		}
+
+		emit energyChanged( m_energy );
+		emit teleportSafelyAllowed( canSafeTeleport() );
+		emit vaporizerAllowed( canUseVaporizer() );
+	}
+}
+
+
+QString Killbots::Engine::gridToString() const
+{
+	QString string;
+	for ( int r = 0; r < m_rules->rows(); ++r )
+	{
+		for ( int c = 0; c < m_rules->columns(); ++c )
+		{
+			switch ( spriteTypeAt( QPoint( c, r ) ) )
+			{
+			case Robot:
+				string += 'r';
+				break;
+			case Fastbot:
+				string += 'f';
+				break;
+			case Junkheap:
+				string += 'j';
+				break;
+			default:
+				string += ' ';
+				break;
+			}
+		}
+		string += '\n';
+	}
+	return string;
 }
