@@ -31,11 +31,6 @@
 static const char description[] =
 	I18N_NOOP("Tactical one or two player game");
 
-// A hack to circumvent tricky i18n issue, not used later on in the code.
-// Both context and contents must be exactly the same as for the entry in
-// kdelibs/kdeui/ui_standards.rc
-static const char dummy[] = I18N_NOOP2("Menu title", "&Move");
-
 int main(int argc, char *argv[])
 {
     KAboutData aboutData( "kjumpingcube", 0, ki18n("KJumpingCube"),
@@ -48,7 +43,7 @@ int main(int argc, char *argv[])
                       ki18n("Upgrade to KDE4 and SVG artwork support."),
                       "ianw2@optusnet.com.au");
     aboutData.addCredit(ki18n("Eugene Trounev"),
-                      ki18n("Graphics for KDE4.0 version."),
+                      ki18n("Graphics for KDE 4.0 version."),
                       "irs_me@hotmail.com");
     KCmdLineArgs::init( argc, argv, &aboutData );
 
