@@ -4,6 +4,8 @@
     email                : roman@sbrf.barrt.ru
     copyright            : (C) 2000 by Roman Razilov
     email                : Roman.Razilov@gmx.de
+    copyright            : (C) 2006 by Dmitry Suzdalev
+    email                : dimsuz@gmail.com
  ***************************************************************************/
 
 /***************************************************************************
@@ -18,28 +20,25 @@
 #ifndef MWIDGET_H
 #define MWIDGET_H
 
-#include <qwidget.h>
-#include <qpixmap.h>
-#include <qtimer.h>
-#include <qframe.h>
-#include "linesboard.h"
-#include "prompt.h"
+#include <QWidget>
 
-class BallPainter;
+class KLinesScene;
+class QLabel;
 
-class MainWidget : public QFrame
+class MainWidget : public QWidget
 {
-  Q_OBJECT
-  LinesBoard * lsb;
-  LinesPrompt * lPrompt;
-  BallPainter *bPainter;
-
+    Q_OBJECT
 public:
-  MainWidget( QWidget* parent=0, const char* name=0 );
-  ~MainWidget();
-  LinesBoard * GetLsb();
-  LinesPrompt * GetPrompt();
-  void updatePix();
+    explicit MainWidget( QWidget* parent=0 );
+    ~MainWidget();
+
+    KLinesScene* scene() { return m_scene; }
+public slots:
+    void setShowNextColors(bool);
+private:
+    virtual void resizeEvent( QResizeEvent* );
+    KLinesScene* m_scene;
+    QLabel *m_next_label;
 };
 
 #endif
