@@ -20,28 +20,21 @@
 
 #include "card.h"
 
-#include <math.h>
-#include <assert.h>
-
-#include <qpainter.h>
-//Added by qt3to4:
-#include <QPixmap>
-#include <QBrush>
-#include <QTimeLine>
-#include <QGraphicsItemAnimation>
-#include <QGraphicsSceneMouseEvent>
-#include <QStyleOptionGraphicsItem>
-#include <qgraphicssvgitem.h>
-#include <QPixmapCache>
-#include <kdebug.h>
-
-#include "pile.h"
 #include "cardmaps.h"
 #include "dealer.h"
+#include "pile.h"
 
+#include <KDebug>
+
+#include <QtCore/QTimeLine>
+#include <QtGui/QGraphicsItemAnimation>
+#include <QtGui/QGraphicsSceneMouseEvent>
+#include <QtGui/QPainter>
+#include <QtGui/QPixmap>
+#include <QtGui/QStyleOptionGraphicsItem>
+
+#include <cmath>
 
-// Run time type id
-const int Card::my_type = DealerScene::CardTypeId;
 
 AbstractCard::AbstractCard( Rank r, Suit s )
     : m_suit( s ), m_rank( r ), m_faceup( false )
@@ -203,8 +196,7 @@ void Card::setZValue(double z)
 //
 void Card::moveTo(qreal x2, qreal y2, qreal z2, int duration)
 {
-
-    //kDebug() << "Card::moveTo" << x2 << " " << y2 << " " << duration << " " << kBacktrace();
+    //kDebug(11111) << "Card::moveTo" << x2 << " " << y2 << " " << duration << " " << kBacktrace();
     if ( fabs( x2 - x() ) < 2 && fabs( y2 - y() ) < 1 )
     {
         setPos( QPointF( x2, y2 ) );
@@ -213,7 +205,7 @@ void Card::moveTo(qreal x2, qreal y2, qreal z2, int duration)
         return;
     }
     stopAnimation();
-    m_isSeen = CardVisible; // avoid suprises
+    m_isSeen = CardVisible; // avoid surprises
 
     QTimeLine *timeLine = new QTimeLine( 1000, this );
 
@@ -230,7 +222,7 @@ void Card::moveTo(qreal x2, qreal y2, qreal z2, int duration)
     timeLine->setDuration( duration );
     timeLine->start();
 
-    connect( timeLine, SIGNAL( finished() ), SLOT( stopAnimation() ) );
+    connect( timeLine, SIGNAL(finished()), SLOT(stopAnimation()) );
 
     m_destX = x2;
     m_destY = y2;
@@ -261,12 +253,17 @@ void Card::flipTo(qreal x2, qreal y2, int duration)
     animation->setScaleAt( 0, 1, 1 );
     animation->setScaleAt( 0.5, 0.0, 1 );
     animation->setScaleAt( 1, 1, 1 );
+
     QPointF hp = pos();
     hp.setX( ( x1 + x2 + boundingRect().width() ) / 2 );
-    //kDebug() << "flip" << name() << " " << x1 << " " << x2 << " " << y1 << " " << y2;
     if ( fabs( y1 - y2) > 2 )
         hp.setY( ( y1 + y2 + boundingRect().height() ) / 20 );
-    //kDebug() << "hp" << pos() << " " << hp << " " << QPointF( x2, y2 );
+    else
+        // Workaround to bug 194775, pending a fix to Qt issue #255469. This
+        // adds a bit of arc to the flip animation, to avoid clipping that
+        // can occur during purely horizontal transformations/translations.
+        hp.ry() -= 5;
+
     animation->setPosAt(0.5, hp );
     animation->setPosAt(1, QPointF( x2, y2 ));
 
@@ -276,8 +273,8 @@ void Card::flipTo(qreal x2, qreal y2, int duration)
     timeLine->setDuration( duration );
     timeLine->start();
 
-    connect( timeLine, SIGNAL( finished() ), SLOT( stopAnimation() ) );
-    connect( timeLine, SIGNAL( valueChanged( qreal ) ), SLOT( flipAnimationChanged( qreal ) ) );
+    connect( timeLine, SIGNAL(finished()), SLOT(stopAnimation()) );
+    connect( timeLine, SIGNAL(valueChanged(qreal)), SLOT(flipAnimationChanged(qreal)) );
 
     // Set the target of the animation
     m_destX = x2;
@@ -295,7 +292,7 @@ void Card::flipAnimationChanged( qreal r)
 {
     if ( r > 0.5 && !isFaceUp() ) {
         flip();
-        assert( m_destFace == m_faceup );
+        Q_ASSERT( m_destFace == m_faceup );
     }
 }
 
@@ -321,7 +318,7 @@ void Card::testVisibility()
             return;
         }
 
-        //kDebug() << c->name() << "covers" << name() << " " << c->mapToScene( c->boundingRect() ).boundingRect() << " " << mapToScene( boundingRect() ).boundingRect() << " " << zValue() << " " << c->zValue();
+        //kDebug(11111) << c->name() << "covers" << name() << " " << c->mapToScene( c->boundingRect() ).boundingRect() << " " << mapToScene( boundingRect() ).boundingRect() << " " << zValue() << " " << c->zValue();
         c->m_hiddenCards.append( this );
         m_isSeen = CardHidden;
         return;
@@ -344,7 +341,7 @@ void Card::stopAnimation()
     if ( !animation )
         return;
 
-    //kDebug() << gettime() << "stopAnimation" << name();
+    //kDebug(11111) << gettime() << "stopAnimation" << name();
     QGraphicsItemAnimation *old_animation = animation;
     animation = 0;
     if ( old_animation->timeLine()->state() == QTimeLine::Running )
@@ -357,7 +354,7 @@ void Card::stopAnimation()
         setSpread( source()->cardOffset(this) );
 
     m_isSeen = Unknown;
-    emit stoped( this );
+    emit stopped( this );
 }
 
 bool  Card::animated() const
@@ -376,7 +373,7 @@ void Card::hoverEnterEvent ( QGraphicsSceneHoverEvent *  )
 
     m_hoverTimer->start(200);
     //zoomIn(400);
-    //kDebug() << "hoverEnterEvent\n";
+    //kDebug(11111) << "hoverEnterEvent\n";
 }
 
 void Card::hoverLeaveEvent ( QGraphicsSceneHoverEvent * )
@@ -393,13 +390,13 @@ void Card::hoverLeaveEvent ( QGraphicsSceneHoverEvent * )
 }
 
 void Card::mousePressEvent ( QGraphicsSceneMouseEvent *ev ) {
-    //kDebug() << "mousePressEvent\n";
+    //kDebug(11111) << "mousePressEvent\n";
     if ( !isFaceUp() )
         return;
     if ( this == source()->top() )
         return; // no way this is meaningful
 
-    if (ev->buttons() & Qt::RightButton && !animated() && !source()->dscene()->isMoving( this ) )
+    if ( ev->button() == Qt::RightButton && !animated() && !source()->dscene()->isMoving( this ) )
     {
         m_hoverTimer->stop();
         stopAnimation();
@@ -410,8 +407,8 @@ void Card::mousePressEvent ( QGraphicsSceneMouseEvent *ev ) {
     }
 }
 
-void Card::mouseReleaseEvent ( QGraphicsSceneMouseEvent * ) {
-    //kDebug() << "mouseReleaseEvent\n";
+void Card::mouseReleaseEvent ( QGraphicsSceneMouseEvent * ev ) {
+    //kDebug(11111) << "mouseReleaseEvent\n";
     m_moving = false;
 
     if ( !isFaceUp() )
@@ -419,7 +416,7 @@ void Card::mouseReleaseEvent ( QGraphicsSceneMouseEvent * ) {
     if ( this == source()->top() )
         return; // no way this is meaningful
 
-    if (m_isZoomed)
+    if ( m_isZoomed && ev->button() == Qt::RightButton )
     {
         m_hoverTimer->stop();
         stopAnimation();
@@ -443,7 +440,7 @@ void Card::getUp()
     timeLine->setDuration( 1500 );
     timeLine->start();
 
-    connect( timeLine, SIGNAL( finished() ), SLOT( stopAnimation() ) );
+    connect( timeLine, SIGNAL(finished()), SLOT(stopAnimation()) );
 
     m_destZ = zValue();
     m_destX = x();
@@ -470,7 +467,7 @@ void Card::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
     Q_ASSERT( !pixmap().isNull() );
 
     //painter->drawPixmap(exposed, pixmap(), exposed );
-    //kDebug() << "exposed" << exposed;
+    //kDebug(11111) << "exposed" << exposed;
 
     QRect body = cardMap::self()->opaqueRect();
     QMatrix m = painter->combinedMatrix();
@@ -532,6 +529,7 @@ void Card::zoomIn(int t)
 
     QTimeLine *timeLine = new QTimeLine( t, this );
     m_originalPosition = pos();
+
     animation = new QGraphicsItemAnimation( this );
     animation->setItem( this );
     animation->setTimeLine( timeLine );
@@ -551,7 +549,7 @@ void Card::zoomIn(int t)
 
     m_isZoomed = true;
 
-    connect( timeLine, SIGNAL( finished() ), SLOT( stopAnimation() ) );
+    connect( timeLine, SIGNAL(finished()), SLOT(stopAnimation()) );
 
     m_destZ = zValue();
     m_destX = dest.x();
@@ -582,7 +580,7 @@ void Card::zoomOut(int t)
 
     m_isZoomed = false;
 
-    connect( timeLine, SIGNAL( finished() ), SLOT( stopAnimation() ) );
+    connect( timeLine, SIGNAL(finished()), SLOT(stopAnimation()) );
 
     m_destZ = zValue();
     m_destX = m_originalPosition.x();
