@@ -11,21 +11,23 @@
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
 */
-#ifndef _DEALER_H_
-#define _DEALER_H_
 
-#include "pile.h"
+#ifndef DEALER_H
+#define DEALER_H
+
 #include "card.h"
-
-#include <krandomsequence.h>
-#include <QGraphicsScene>
-
-class MoveHint;
-class QDomDocument;
 class DealerInfo;
 class DealerScene;
+class MoveHint;
+#include "pile.h"
 class Solver;
 
+#include <KRandomSequence>
+
+#include <QGraphicsScene>
+class QDomDocument;
+
+
 typedef QList<Pile*> PileList;
 
 class DealerInfoList {
@@ -44,15 +46,17 @@ class DealerInfo
 {
 public:
     DealerInfo(const char *_name, int _index)
-        : name(_name),
-	gameindex(_index)
+        : name(_name)
 	{
 	    DealerInfoList::self()->add(this);
+            addOldId(_index);
 	}
-	virtual ~DealerInfo(){}
-	const char *name;
-	int gameindex;
-	virtual DealerScene *createGame() = 0;
+    virtual ~DealerInfo(){}
+    const char *name;
+    QList<int> ids;
+    virtual DealerScene *createGame() const = 0;
+    void addOldId(int id) { ids.push_back(id); }
+    bool hasId(int id) { return ids.contains(id); }
 };
 
 class CardState;
@@ -70,33 +74,32 @@ class DealerScene : public QGraphicsScene
     Q_OBJECT
 
 public:
-
-    enum UserTypes { CardTypeId = 1, PileTypeId = 2 };
-
     DealerScene();
     ~DealerScene();
+
     void unmarkAll();
     void mark(Card *c);
+    QString save_it();
+    Pile * targetPile();
 
 protected:
     virtual void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent );
     virtual void mouseMoveEvent ( QGraphicsSceneMouseEvent * mouseEvent );
     virtual void mousePressEvent ( QGraphicsSceneMouseEvent * mouseEvent );
     virtual void mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    void countLoss();
-    void countGame();
     void drawBackground ( QPainter * painter, const QRectF & rect );
 
 public:
     Pile *findTarget(Card *c);
     virtual bool cardClicked(Card *);
-    virtual void pileClicked(Pile *);
+    virtual bool pileClicked(Pile *);
     virtual bool cardDblClicked(Card *);
 
     bool isMoving(Card *c) const;
 
     void saveGame(QDomDocument &doc);
     void openGame(QDomDocument &doc);
+    virtual void mapOldId(int id);
 
     virtual void getHints();
     void getSolverHints();
@@ -105,14 +108,15 @@ public:
     // it's not const because it changes the random seed
     virtual MoveHint *chooseHint();
 
+    void setSolverEnabled(bool a);
     void setAutoDropEnabled(bool a);
     bool autoDrop() const { return _autodrop; }
 
     // use this for autodrop times
     int speedUpTime( int delay ) const;
 
-    void setGameNumber(long gmn);
-    long gameNumber() const;
+    void setGameNumber(int gmn);
+    int gameNumber() const;
 
     int waiting() const { return _waiting; }
     void setWaiting(bool w);
@@ -126,8 +130,6 @@ public:
     void setGameId(int id);
     int gameId() const;
 
-    void startNew();
-
     virtual void restart() = 0;
     virtual void stopDemo();
 
@@ -135,20 +137,17 @@ public:
     virtual bool checkAdd   ( int checkIndex, const Pile *c1, const CardList& c2) const;
     virtual bool checkPrefering( int checkIndex, const Pile *c1, const CardList& c2) const;
 
-    // reimplement this to add game-specific information in the state structure
-    virtual QString getGameState() const { return QString(); }
-    // reimplement this to use the game-specific information from the state structure
-    virtual void setGameState( const QString & ) {}
-
     QPointF maxPilePos() const;
+    qreal offsetX() const;
+    qreal offsetY() const;
 
     void setSceneSize( const QSize &s );
 
     void won();
     bool demoActive() const;
-    int getMoves() const { return undoList.count(); }
+    int getMoves() const;
 
-    enum { None = 0, Hint = 1, Demo = 2, Redeal = 4 } Actions;
+    enum { None = 0, Hint = 1, Demo = 2, Draw = 4, Deal = 8, Redeal = 16 } Actions;
 
     void setActions(int actions);
     int actions() const;
@@ -156,6 +155,7 @@ public:
     void setSolver( Solver *s);
     Solver *solver() const;
 
+    void startSolver() const;
     void unlockSolver() const;
     void finishSolver() const;
 
@@ -163,22 +163,35 @@ public:
     int neededFutureMoves() const;
 
     bool isInitialDeal() const;
+    void recordGameStatistics();
 
     bool cardsAreMoving() const { return !movingCards.empty(); }
 
+    void createDump( QPaintDevice * );
+
+    void updateWonItem();
+
+    bool allowedToStartNewGame();
+
 public slots:
     virtual bool startAutoDrop();
+    virtual Card *newCards();
     void hint();
     void rescale(bool onlypiles);
 
     State *getState();
     void setState(State *);
     void relayoutPiles();
-    void slotShowGame(bool);
+    void showWonMessage();
     void takeState();
+    void eraseRedo();
     void undo();
+    void redo();
     void slotSolverEnded();
     void slotSolverFinished();
+    void slotAutoDrop();
+
+    void startNew(int gameNumber = -1);
 
 protected slots:
     virtual void demo();
@@ -188,16 +201,17 @@ protected slots:
 
 signals:
     void undoPossible(bool poss);
+    void redoPossible(bool poss);
     void hintPossible(bool poss);
     void demoPossible(bool poss);
-    void redealPossible(bool poss);
+    void newCardsPossible(bool poss);
 
-    void gameInfo(const QString &info);
     void gameWon(bool withhelp);
     void demoActive(bool en);
-    void updateMoves();
+    void updateMoves(int moves);
     void gameLost();
 
+    void gameSolverReset();
     void gameSolverStart();
     void gameSolverWon();
     void gameSolverLost();
@@ -208,24 +222,32 @@ private slots:
     void stopAndRestartSolver();
 
 protected:
+    // reimplement these to store and load game-specific information in the state structure
+    virtual QString getGameState() { return QString(); }
+    virtual void setGameState( const QString & ) {}
+
+    // reimplement these to store and load game-specific options in the saved game file
+    virtual QString getGameOptions() const { return QString(); }
+    virtual void setGameOptions( const QString & ) {}
+
+    virtual void newDemoMove(Card *m);
+    void considerGameStarted();
+
     PileList piles;
     QList<MoveHint*> hints;
     KRandomSequence randseq;
 
-    virtual Card *demoNewCards();
-    virtual void newDemoMove(Card *m);
-
 private:
     QList<QGraphicsItem *> marked;
-    QList<State*> undoList;
 
     bool moved;
     CardList movingCards;
 
     QPointF moving_start;
     bool _autodrop;
+    bool _usesolver;
     int _waiting;
-    long gamenumber;
+    int gamenumber;
 
     class DealerScenePrivate;
     DealerScenePrivate *d;
