@@ -13,14 +13,14 @@
 */
 
 #include "fortyeight.h"
+
 #include "deck.h"
 #include "speeds.h"
 #include "patsolve/fortyeight.h"
 
-#include <cassert>
+#include <KDebug>
+#include <KLocale>
 
-#include <klocale.h>
-#include <kdebug.h>
 
 HorLeftPile::HorLeftPile( int _index, DealerScene* parent)
     : Pile(_index, parent)
@@ -51,7 +51,7 @@ Fortyeight::Fortyeight( )
 
     const qreal dist_x = 11.1;
 
-    connect(Deck::deck(), SIGNAL(pressed(Card*)), SLOT(deckPressed(Card*)));
+    connect(Deck::deck(), SIGNAL(pressed(Card*)), SLOT(newCards()));
     connect(Deck::deck(), SIGNAL(clicked(Card*)), SLOT(deckClicked(Card*)));
     Deck::deck()->setPilePos( -2, -1);
     Deck::deck()->setZValue(20);
@@ -79,7 +79,7 @@ Fortyeight::Fortyeight( )
         stack[i]->setReservedSpace( QSizeF( 10, 40 ) );
     }
 
-    setActions(DealerScene::Hint | DealerScene::Demo);
+    setActions(DealerScene::Hint | DealerScene::Demo | DealerScene::Draw);
     setSolver( new FortyeightSolver( this ) );
 }
 
@@ -90,33 +90,37 @@ void Fortyeight::restart()
     lastdeal = false;
     Deck::deck()->collectAndShuffle();
     deal();
+    emit newCardsPossible( true );
 }
 
 void Fortyeight::deckClicked( Card * )
 {
-    //kDebug() << "deckClicked" << c->name() << " " << pile->top()->name() << " " << pile->top()->animated();
+    //kDebug(11111) << "deckClicked" << c->name() << " " << pile->top()->name() << " " << pile->top()->animated();
     if ( pile->top() && pile->top()->animated() )
         return;
     if ( Deck::deck()->isEmpty())
-        deckPressed( 0 );
+        newCards();
 }
 
-void Fortyeight::deckPressed(Card *c2)
+Card *Fortyeight::newCards()
 {
-    if( c2 )
-        kDebug(11111) << gettime() << "deckPressed" << c2->rank() << " " << c2->suit();
-    else
-        kDebug(11111) << gettime() << "deckPressed" << "(nil)";
-    if (Deck::deck()->isEmpty()) {
-        if (lastdeal)
-            return;
+    if (Deck::deck()->isEmpty() && lastdeal)
+        return 0;
+
+    if (pile->top() && pile->top()->animated())
+        return pile->top();
+
+    if (Deck::deck()->isEmpty())
+    {
         lastdeal = true;
-        while (!pile->isEmpty()) {
+        while (!pile->isEmpty())
+        {
             Card *c = pile->at(pile->cardsLeft()-1);
             c->stopAnimation();
             Deck::deck()->add(c, true);
         }
     }
+
     Card *c = Deck::deck()->nextCard();
     pile->add(c, true);
     c->stopAnimation();
@@ -124,15 +128,13 @@ void Fortyeight::deckPressed(Card *c2)
     qreal y = c->realY();
     c->setPos( Deck::deck()->pos() );
     c->flipTo(x, y, DURATION_FLIP );
-    takeState();
-}
 
-Card *Fortyeight::demoNewCards()
-{
-    if (Deck::deck()->isEmpty() && lastdeal)
-        return 0;
-    deckPressed(0);
-    return pile->top();
+    takeState();
+    considerGameStarted();
+    if ( Deck::deck()->isEmpty() && lastdeal )
+        emit newCardsPossible( false );
+
+    return c;
 }
 
 bool Fortyeight::checkAdd(int, const Pile *c1, const CardList &c2) const
@@ -162,7 +164,7 @@ void Fortyeight::deal()
     pile->add(Deck::deck()->nextCard(), false);
 }
 
-QString Fortyeight::getGameState() const
+QString Fortyeight::getGameState()
 {
     return QString::number(lastdeal);
 }
@@ -170,13 +172,14 @@ QString Fortyeight::getGameState() const
 void Fortyeight::setGameState( const QString &s )
 {
     lastdeal = s.toInt();
+    emit newCardsPossible( !lastdeal || !Deck::deck()->isEmpty() );
 }
 
 static class LocalDealerInfo8 : public DealerInfo
 {
 public:
-    LocalDealerInfo8() : DealerInfo(I18N_NOOP("Forty && Eight"), 8) {}
-    virtual DealerScene *createGame() { return new Fortyeight(); }
+    LocalDealerInfo8() : DealerInfo(I18N_NOOP("Forty & Eight"), 8) {}
+    virtual DealerScene *createGame() const { return new Fortyeight(); }
 } ldi9;
 
 //-------------------------------------------------------------------------//
