@@ -11,24 +11,37 @@
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
 */
-#ifndef GAMESTATS_IMPL_H_
-#define GAMESTATS_IMPL_H_
 
-#include <qdialog.h>
+#ifndef GAMESTATSIMPL_H
+#define GAMESTATSIMPL_H
+
 #include "ui_gamestats.h"
 
-class GameStatsImpl : public QDialog, Ui::GameStats
+#include <KDialog>
+
+#include <QtCore/QMap>
+
+
+class GameStatsImpl : public KDialog
 {
-    Q_OBJECT
+	Q_OBJECT
 
 	public:
 		explicit GameStatsImpl(QWidget* aParent);
 
-		void showGameType(int i);
-        public slots:
-		void setGameType(int i);
+		void showGameType(int gameIndex);
+	public slots:
+		void setGameType(int gameIndex);
 		void resetStats();
+
+	private slots:
+		void selectionChanged(int comboIndex);
+
+	private:
+		Ui::GameStats * ui;
+		QMap<int,int> indexToIdMap;
 };
 
 #endif
 
+// kate: replace-tabs off; replace-tabs-save off
