@@ -204,8 +204,7 @@ void Klondike::restart()
 
 void Klondike::gameTypeChanged()
 {
-    if ( demoActive() || isGameWon()  )
-       return;
+    stopDemo();
 
     if ( allowedToStartNewGame() )
     {
