@@ -1,10 +1,9 @@
-
-#ifndef _CLOCKSOLVER_H
-#define _CLOCKSOLVER_H
-
-#include "patsolve.h"
+#ifndef CLOCKSOLVER_H
+#define CLOCKSOLVER_H
 
 class Clock;
+#include "patsolve.h"
+
 
 class ClockSolver : public Solver
 {
@@ -24,5 +23,5 @@ public:
 
     const Clock *deal;
 };
-#endif
 
+#endif // CLOCKSOLVER_H
