@@ -1,16 +1,11 @@
 /* Common routines & arrays. */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <sys/types.h>
-#include <stdarg.h>
 #include "freecell.h"
+
 #include "../freecell.h"
 #include "../pile.h"
 
+
 /* Some macros used in get_possible_moves(). */
 
 /* The following macro implements
