@@ -1,10 +1,9 @@
-
-#ifndef _GRANDFSOLVER_H
-#define _GRANDFSOLVER_H
-
-#include "patsolve.h"
+#ifndef GRANDFSOLVER_H
+#define GRANDFSOLVER_H
 
 class Grandf;
+#include "patsolve.h"
+
 
 class GrandfSolver : public Solver
 {
@@ -29,5 +28,5 @@ public:
     int m_redeal;
     int offs;
 };
-#endif
 
+#endif // GRANDFSOLVER_H
