@@ -1,10 +1,9 @@
-
-#ifndef _MOD3SOLVER_H
-#define _MOD3SOLVER_H
-
-#include "patsolve.h"
+#ifndef MOD3SOLVER_H
+#define MOD3SOLVER_H
 
 class Mod3;
+#include "patsolve.h"
+
 
 class Mod3Solver : public Solver
 {
@@ -26,5 +25,5 @@ public:
     int aces;
     int deck;
 };
-#endif
 
+#endif // MOD3SOLVER_H
