@@ -1,28 +1,22 @@
 /* Common routines & arrays. */
 
 #include "simon.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <kdebug.h>
-#include <sys/types.h>
-#include <stdarg.h>
-#include "simon.h"
-#include "../simon.h"
-#include "../pile.h"
-#include "../deck.h"
 
-/* These two routines make and unmake moves. */
+#include "../deck.h"
+#include "../pile.h"
+#include "../simon.h"
+
+#include <KDebug>
+
 
 #define PRINT 0
-#define PRINT2 0
+
+/* These two routines make and unmake moves. */
 
 void SimonSolver::make_move(MOVE *m)
 {
 #if PRINT
-    kDebug() << "\n\nmake_move\n";
+    kDebug(11111) << "\n\nmake_move\n";
     if ( m->totype == O_Type )
         fprintf( stderr, "move %d from %d out (at %d) Prio: %d\n\n", m->card_index, m->from, m->turn_index, m->pri );
     else
@@ -76,7 +70,7 @@ void SimonSolver::make_move(MOVE *m)
 void SimonSolver::undo_move(MOVE *m)
 {
 #if PRINT
-    kDebug() << "\n\nundo_move\n";
+    kDebug(11111) << "\n\nundo_move\n";
     if ( m->totype == O_Type )
         fprintf( stderr, "move %d from %d out (at %d)\n\n", m->card_index, m->from, m->turn_index );
     else
@@ -418,7 +412,7 @@ MoveHint *SimonSolver::translateMove( const MOVE &m )
     Pile *frompile = deal->store[m.from];
     Card *card = frompile->at( frompile->cardsLeft() - m.card_index - 1);
 
-    kDebug() << "card" << card->rank() << " " << card->suit();
+    kDebug(11111) << "card" << card->rank() << " " << card->suit();
     if ( m.totype == O_Type )
     {
         for ( int i = 0; i < 4; i++ )
