@@ -15,13 +15,10 @@
 #define PILE_H
 
 #include "card.h"
-
-#include <QPixmap>
-//Added by qt3to4:
-#include <QGraphicsPixmapItem>
-
 class DealerScene;
 
+#include <QtGui/QGraphicsPixmapItem>
+
 
 /***************************************
 
@@ -65,7 +62,7 @@ public:
     bool legalRemove(const Card *c, bool demo = false) const;
 
     virtual void moveCards(CardList &c, Pile *to = 0);
-    void moveCardsBack(CardList &c, int steps = -1 );
+    void moveCardsBack(CardList& c, int duration = -1);
 
     void setRemoveFlags( int flag ) { removeFlags = flag; }
     void setAddFlags( int flag ) { addFlags = flag; }
@@ -79,6 +76,9 @@ public:
     void setHighlighted( bool flag );
     bool isHighlighted() const { return m_highlighted; }
 
+    void setGraphicVisible( bool flag );
+    bool isGraphicVisible() { return m_graphicVisible; } ;
+
     CardList cardPressed(Card *c);
 
     Card *top() const;
@@ -91,9 +91,8 @@ public:
     int  index()   const { return myIndex; }
     bool isEmpty() const { return m_cards.count() == 0; }
 
-    static const int my_type;
-
-    virtual int  type() const       { return UserType + my_type; }
+    enum { Type = UserType + 2 };
+    virtual int type() const { return Type; }
 
     virtual void setVisible(bool vis);
 
@@ -131,13 +130,11 @@ public:
 
     void rescale();
 
-    static QSvgRenderer *pileRenderer();
-
     void setReservedSpace( const QSizeF &p) { m_reserved = p; }
     QSizeF reservedSpace() const { return m_reserved; }
 
-    void setMaximalSpace( const QSizeF &s) { m_space = s; }
-    QSizeF maximalSpace() const { return m_space; }
+    void setMaximumSpace( const QSizeF &s) { m_space = s; }
+    QSizeF maximumSpace() const { return m_space; }
 
     void tryRelayoutCards();
 
@@ -153,14 +150,14 @@ signals:
     void dblClicked(Card *c);
 
 protected:
+    virtual void layoutCards(int duration);
+
     int       removeFlags;
     int       addFlags;
     CardList  m_cards;
     QTimer *m_relayoutTimer;
 
 private:
-    static QSvgRenderer *_renderer;
-
     PileType  _atype;
     PileType  _rtype;
     double    _spread;
@@ -169,13 +166,11 @@ private:
     int myIndex;
     bool _target;
 
-    static QPixmap *cache;
-    static QPixmap *cache_selected;
     bool m_highlighted;
+    bool m_graphicVisible;
     QPointF _pilePos;
     QSizeF m_reserved;
     QSizeF m_space;
-
 };
 
 #endif
