@@ -150,14 +150,7 @@ Spider::Spider()
 
 void Spider::gameTypeChanged()
 {
-    if ( waiting() )
-    {
-        QTimer::singleShot( 100, this, SLOT( gameTypeChanged() ) );
-        return;
-    }
-
-    if ( demoActive() || isGameWon()  )
-        return;
+    stopDemo();
 
     if ( allowedToStartNewGame() )
     {
