@@ -30,6 +30,7 @@
 #include "Sprites/cannonsprite.h"
 #include "Sprites/cavalrysprite.h"
 #include "Sprites/infantrysprite.h"
+#include "Sprites/arrowsprite.h"
 #include "Sprites/skinSpritesData.h"
 #include "GameLogic/KMessageParts.h"
 #include "GameLogic/onu.h"
@@ -171,7 +172,7 @@ AnimSprite* KGameWindow::initArmiesMultipleCombat(unsigned int nbA,
   {
     int relativePosInArena=0;
 
-    if (firstCountry->id() == secondCountry->id())
+    if (firstCountry->name() == secondCountry->name())
     {
       sprite->setDefendant();
       relativePosInArena = relativePosInArenaDefense;
@@ -217,11 +218,18 @@ QPointF KGameWindow::determinePointDepartArena(Country *pays, int relativePos)
 }
 
 
-void KGameWindow::determinePointArrivee(Country *attackingCountry,
-        Country *defendingCountry,
+void KGameWindow::determinePointArrivee(
         QPointF& pointArriveeAttaquant,
         QPointF& pointArriveeDefenseur)
 {
+  Country *attackingCountry = firstCountry();
+  Country *defendingCountry = secondCountry();
+  if (attackingCountry == 0 || m_secondCountry == 0)
+  {
+    kError() << "attackingCountry=" << (void*)attackingCountry << "defendingCountry=" << (void*)defendingCountry;
+    return;
+  }
+  
   //  - attacker's flag point
   qreal pointFlagAttaquantX = attackingCountry-> pointFlag().x()*m_theWorld->zoom();
 
@@ -283,7 +291,8 @@ void KGameWindow::determinePointArrivee(Country *attackingCountry,
   }
 
   // vertical meet point
-  pointArriveeY = (((defendingCountry-> pointFlag().y() + Sprites::SkinSpritesData::single().intData("fighters-flag-y-diff")))* m_theWorld->zoom()) ;
+  int diff = Sprites::SkinSpritesData::single().intData("flag-height") - Sprites::SkinSpritesData::single().intData("cannon-height");
+  pointArriveeY = (((defendingCountry-> pointFlag().y() + diff))* m_theWorld->zoom()) ;
   
   kDebug() << "2 " << defendingCountry-> pointFlag().y() << pointArriveeY;
   if (!attackingCountry->communicateWith(defendingCountry))
@@ -353,12 +362,13 @@ void KGameWindow::determinePointArrivee(Country *attackingCountry,
 
 // point arriveedefenseur inutile
 void KGameWindow::determinePointArriveeForArena(
-    Country *attackingCountry,
-    Country *defendingCountry,
     int relative,
     QPointF& pointArriveeAttaquant,
     QPointF& pointArriveeDefenseur)
 {
+  Country *attackingCountry = firstCountry();
+  Country *defendingCountry = secondCountry();
+  
   kDebug() << m_firstCountry->name() << "("<<(void*)m_firstCountry<<")"
       << m_secondCountry->name() << "("<<(void*)m_secondCountry<<")";
   kDebug() << attackingCountry->name() << "("<<(void*)attackingCountry<<")"
@@ -523,17 +533,25 @@ void KGameWindow::determinePointArriveeForArena(
 }
 
 
-void KGameWindow::initCombatMovement(
-    Country *attackingCountry,
-    Country *defendingCountry)
+void KGameWindow::initCombatMovement()
 {
   kDebug() << "1";
 
-  gameActionsToolBar-> hide();
-
+  Country *attackingCountry = firstCountry();
+  Country *defendingCountry = secondCountry();
+  if (attackingCountry == 0 || defendingCountry == 0)
+  {
+    kError() << "attackingCountry=" << (void*)attackingCountry << "defendingCountry=" << (void*)defendingCountry;
+    return;
+  }
+  
   getRightDialog()->close();
-  getRightDialog()->displayFightDetails(firstCountry(), secondCountry(),
-      firstCountry()->owner()->getNbAttack(), secondCountry()->owner()->getNbDefense());
+  if (firstCountry() != 0 && firstCountry()->owner() != 0 
+    && secondCountry() != 0 && secondCountry()->owner() != 0)
+  {
+    getRightDialog()->displayFightDetails(firstCountry(), secondCountry(),
+        firstCountry()->owner()->getNbAttack(), secondCountry()->owner()->getNbDefense());
+  }
   centerOnFight();  //center the game on the fight
   if  (isArena()
       && ! m_automaton->currentPlayer()->isAI()
@@ -548,21 +566,20 @@ void KGameWindow::initCombatMovement(
   m_animFighters->changeTarget(this,SLOT(slotMovingFightersArrived(AnimSpritesGroup*)));
 
   QString sndRoulePath;
-  AnimSprite* defenderSprite=0;
-  AnimSprite* attackingSprite=0;
+  AnimSprite* defenderSprite = 0;
+  AnimSprite* attackingSprite = 0;
 
   QPointF pointArriveeAttaquant(0,0);
   QPointF pointArriveeDefenseur(0,0);
  
   if (backGnd()->bgIsArena())
   {
-    determinePointArriveeForArena(attackingCountry, defendingCountry, 0,
+    determinePointArriveeForArena(0,
         pointArriveeAttaquant, pointArriveeDefenseur);
   }
   else
   {
-    determinePointArrivee(attackingCountry, defendingCountry,
-        pointArriveeAttaquant, pointArriveeDefenseur);
+    determinePointArrivee(pointArriveeAttaquant, pointArriveeDefenseur);
   }
 
   if (!attackingCountry->spritesInfantry().isEmpty()
@@ -677,7 +694,7 @@ void KGameWindow::initCombatMovement(
       sndRoulePath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/roll.wav");
       if (sndRoulePath.isNull())
       {
-        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error !"));
+        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error!"));
         exit(2);
       }
       if (KsirkSettings::soundEnabled())
@@ -764,14 +781,14 @@ void KGameWindow::initCombatMovement(
     }
     else
     {
-      kDebug() << "No sprite on defending country";
-      assert(false);
+      kError() << "No sprite on defending country" << defendingCountry->name();
+      return;
     }
 
     if (defenderSprite==0)
     {
       kError() << "ERROR: null defenderSprite at " << __FILE__ << ", line " << __LINE__;
-//       return;
+      return;
     }
     else
     {
@@ -785,14 +802,14 @@ void KGameWindow::initCombatMovement(
       }
 
       defenderSprite-> setDefendant();
-      defenderSprite-> setupTravel(defendingCountry, attackingCountry, &pointArriveeDefenseur);
+      defenderSprite-> setupTravel(defendingCountry, defendingCountry, &pointArriveeDefenseur);
       //(pointDepartDefenseurX <= pointArriveeDefenseurX) ? defenderSprite-> setLookRight() : defenderSprite-> setLookLeft();
       kDebug() << "add a sprite 3";
       m_animFighters->addSprite(defenderSprite);
       sndRoulePath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/roll.wav");
       if (sndRoulePath.isNull())
       {
-        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error !"));
+        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error!"));
         exit(2);
       }
       if (KsirkSettings::soundEnabled())
@@ -837,7 +854,7 @@ void KGameWindow::animCombat()
     if (sndCanonPath.isNull())
     {
       KMessageBox::error(0,
-          i18n("Sound cannon not found - Verify your installation<br>Program cannot continue"), i18n("Error !"));
+          i18n("Sound cannon not found - Verify your installation<br>Program cannot continue"), i18n("Error!"));
       exit(2);
     }
     if (KsirkSettings::soundEnabled())
@@ -874,10 +891,11 @@ void KGameWindow::stopCombat()
 }
 
 
-void KGameWindow::animExplosion(int who,Country *attackingCountry, Country *defendingCountry)
+void KGameWindow::animExplosion(int who)
 {
   kDebug() << who;
-
+  Country *attackingCountry = firstCountry();
+  
   m_animFighters->changeTarget(this, SLOT(slotExplosionFinished(AnimSpritesGroup*)));
 
   kDebug() << m_animFighters->size() << " fighters";
@@ -911,7 +929,7 @@ void KGameWindow::animExplosion(int who,Country *attackingCountry, Country *defe
         QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
         if (sndCrashPath.isNull())
         {
-          KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+          KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
           exit(2);
         }
         if (KsirkSettings::soundEnabled())
@@ -931,7 +949,7 @@ void KGameWindow::animExplosion(int who,Country *attackingCountry, Country *defe
         QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
         if (sndCrashPath.isNull())
         {
-          KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+          KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
           exit(2);
         }
         if (KsirkSettings::soundEnabled())
@@ -971,7 +989,7 @@ void KGameWindow::animExplosion(int who,Country *attackingCountry, Country *defe
   /*QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
   if (sndCrashPath.isNull())
   {
-    KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+    KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
     exit(2);
   }
   if (KsirkSettings::soundEnabled())
@@ -985,10 +1003,13 @@ void KGameWindow::animExplosion(int who,Country *attackingCountry, Country *defe
 }
 
 
-void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defendingCountry)
+void KGameWindow::animExplosionForArena()
 {
   kDebug();
 
+  Country *attackingCountry = firstCountry();
+  Country *defendingCountry = secondCountry();
+
   m_animFighters->changeTarget(this, SLOT(slotExplosionFinished(AnimSpritesGroup*)));
 
   kDebug() << "hidden; " << m_animFighters->size() << " fighters";
@@ -1029,7 +1050,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1066,7 +1087,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1101,7 +1122,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1155,7 +1176,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1192,7 +1213,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1226,7 +1247,7 @@ void KGameWindow::animExplosionForArena(Country *attackingCountry, Country *defe
           QString sndCrashPath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/crash.wav");
           if (sndCrashPath.isNull())
           {
-            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error !"));
+            KMessageBox::information(this, i18n("Sound crash not found - Verify your installation\nProgram cannot continue"), i18n("KsirK - Error!"));
             exit(2);
           }
           if (KsirkSettings::soundEnabled())
@@ -1270,7 +1291,7 @@ void KGameWindow::initCombatBringBackForArena(Country *attackingCountry, Country
   if ((NKD != 0)&&(NKA != 0)) who = 2;
   else if (NKA != 0) who = 0;
   else if (NKD != 0) who = 1;
-  else KMessageBox::information(0, i18n("Problem : no one destroyed"));
+  else KMessageBox::information(0, i18n("Problem: no one destroyed"));
 
   //CannonSprite *newSprite;
   AnimSprite* newSprite;
@@ -1279,7 +1300,7 @@ void KGameWindow::initCombatBringBackForArena(Country *attackingCountry, Country
   qreal pointDepartAttaquantY;
   qreal pointDepartDefenseurY;
 
-  qreal flagYDiff = Sprites::SkinSpritesData::single().intData("fighters-flag-y-diff")*m_theWorld->zoom();
+  qreal flagYDiff = (Sprites::SkinSpritesData::single().intData("flag-height") - Sprites::SkinSpritesData::single().intData("cannon-height"))*m_theWorld->zoom();
   qreal pointFlagX = backGnd()->boundingRect().width()/2;
 
   if ((attackingCountry->nbArmies() % 10) == 0)
@@ -1389,7 +1410,7 @@ void KGameWindow::initCombatBringBackForArena(Country *attackingCountry, Country
     QString sndRoulePath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/roll.wav");
     if (sndRoulePath.isNull())
     {
-        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error !"));
+        KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error!"));
         exit(2);
     }
     if (KsirkSettings::soundEnabled())
@@ -1466,7 +1487,7 @@ void KGameWindow::initCombatBringBackForArena(Country *attackingCountry, Country
       QString sndRoulePath = m_dirs-> findResource("appdata", m_automaton->skin() + "/Sounds/roll.wav");
       if (sndRoulePath.isNull())
       {
-          KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error !"));
+          KMessageBox::error(0, i18n("Sound roule not found - Verify your installation<br>Program cannot continue"), i18n("Error!"));
           exit(2);
       }
       if (KsirkSettings::soundEnabled())
@@ -1527,4 +1548,40 @@ bool KGameWindow::haveAnimFighters() const
     return !m_animFighters->empty();
 }
 
+void KGameWindow::updateScrollArrows()
+{
+  kDebug();
+  if (m_uparrow != 0)
+  {
+    QPointF pos = m_frame->mapToScene(QPoint(m_frame->viewport()->width()/2,0));
+    pos = pos + QPointF(-(m_uparrow->boundingRect().width()/2),m_uparrow->boundingRect().height());
+    m_uparrow->setPos(pos);
+    m_uparrow->setActive(false);
+  }
+  if (m_downarrow != 0)
+  {
+    QPointF pos = m_frame->mapToScene(QPoint(m_frame->viewport()->width()/2,m_frame->viewport()->height()));
+    pos = pos - QPointF(m_downarrow->boundingRect().width()/2,m_downarrow->boundingRect().height());
+    m_downarrow->setPos(pos);
+    m_downarrow->setActive(false);
+  }
+  if (m_leftarrow != 0)
+  {
+    QPointF pos = m_frame->mapToScene(QPoint(0,m_frame->viewport()->height()/2));
+    pos = pos - QPointF(m_downarrow->boundingRect().width()/2,m_downarrow->boundingRect().height());
+    pos = pos + QPointF(m_leftarrow->boundingRect().width(),-(m_leftarrow->boundingRect().height()/2));
+    m_leftarrow->setPos(pos);
+    m_leftarrow->setActive(false);
+  }
+  if (m_rightarrow != 0)
+  {
+    QPointF pos = m_frame->mapToScene(QPoint(m_frame->viewport()->width(),m_frame->viewport()->height()/2));
+    pos = pos - QPointF(m_rightarrow->boundingRect().width(),m_rightarrow->boundingRect().height()/2);
+    m_rightarrow->hide();
+    m_rightarrow->setPos(pos);
+    m_rightarrow->show();
+    m_rightarrow->setActive(false);
+  }
+}
+
 } // closing namespace KsirK
