@@ -28,7 +28,11 @@
 #include "GameLogic/gameautomaton.h"
 #include "GameLogic/player.h"
 #include "GameLogic/country.h"
+#include "Dialogs/InvasionSlider.h"
 #include "Sprites/animspriteslist.h"
+#include "Jabber/jabberclient.h"
+
+#include "qca.h"
 
 // include files for Qt
 #include <QPointF>
@@ -49,20 +53,23 @@
 // include files for kde games
 #include <kgame/kgamechat.h>
 
-#include <vector>
-
 // #include <kdialogbase.h>
 
+class KsirkJabberGameWidget;
+class mainMenu;
+
 class QAction;
 class KGameChat;
 class KGamePopupItem;
-class KToolBar;
 class KDialog;
+class KAction;
 
 class QEvent;
 class QDockWidget;
 class QGraphicsScene;
 
+// class JabberClient;
+
 namespace Phonon
 {
   class MediaObject;
@@ -74,15 +81,19 @@ namespace Ksirk
   // forward declaration of the KsirK classes
   class DecoratedGameFrame;
   class FightArena;
-  class mainMenu;
   class AnimSpritesGroup;
   class KRightDialog;
+  class NewGameDialogImpl;
+  
+namespace Sprites
+{
+  class ArrowSprite;
+}
 
 namespace GameLogic
 {
   class ONU;
   class KMessageParts;
-  class GameAutomaton;
   class Player;
 }
 
@@ -100,9 +111,15 @@ class KGameWindow: public KXmlGuiWindow
 
 public:
   enum MessageShowingType {OnConfig, ForceShowing};
-  enum InvasionType {Invasion, Moving};
   enum FightType {Attack, Defense};
-    
+  enum TabbedWidgetsIndexesType
+  {
+    MAINMENU_INDEX /*0*/,
+    NEWGAME_INDEX /*1*/,
+    JABBERGAME_INDEX /*2*/,
+    MAP_INDEX /*3*/,
+    ARENA_INDEX /*4*/
+  };
   /**
     * Create the window and initializes its members
     */
@@ -138,6 +155,8 @@ public:
     * Loads a new skin.
     */
   void newSkin(const QString& onuDefinitionFileName = "");
+
+  void setStateBeforeNewGame(GameLogic::GameAutomaton::GameState state) {m_stateBeforeNewGame = state;}
   
 /************** METHODS BELOW ARE DEFINED IN gestionSprites.cpp **************/
   /**
@@ -153,14 +172,10 @@ public:
   QPointF determinePointDepartArena(GameLogic::Country *pays, int relativePos);
 
   void determinePointArrivee(
-      GameLogic::Country *paysAttaquant,
-      GameLogic::Country *paysDefenseur,
       QPointF& pointArriveeAttaquant,
       QPointF& pointArriveeDefenseur);
 
   void determinePointArriveeForArena(
-      GameLogic::Country *paysAttaquant,
-      GameLogic::Country *paysDefenseur,
       int relative,
       QPointF& pointArriveeAttaquant,
       QPointF& pointArriveeDefenseur);
@@ -170,8 +185,7 @@ public:
     * Initializes the sprites that will fight for the attacker and the
     * defender. Prepares them for moving
     */
-  void initCombatMovement(GameLogic::Country *attacker, 
-    GameLogic::Country *defender);
+  void initCombatMovement();
     
   /**
     * Prepare the fighting animation : replace the sprites sequence by the
@@ -192,11 +206,9 @@ public:
   /**
     * Replaces the sequence of destroyed cannon(s) by the explosion sequence.
     */
-  void animExplosion(int,GameLogic::Country *attacker, 
-    GameLogic::Country *defender);
+  void animExplosion(int who);
 
-  void animExplosionForArena(GameLogic::Country *attacker, 
-    GameLogic::Country *defender);
+  void animExplosionForArena();
 
   /**
     * When all the explosion sequence has been shown for all explosing
@@ -227,12 +239,12 @@ public:
     * Prepares the players for the game with human interaction. Return true
     * if successful or false if failure or cancel
     */
-  bool setupPlayers();
+  bool setupPlayers(GameLogic::GameAutomaton::NetworkGameType socket);
   bool setupOnePlayer();
   bool setupOneWaitedPlayer();
   bool createWaitedPlayer(quint32 waitedPlayerId);
     
-/**
+  /**
     * Do the distribution of armies for all players in function of their
     * number of countries
     */
@@ -262,18 +274,13 @@ public:
     * Display and removes various toolbar buttons in function of the state
     * of the game.
     */
-  void displayOpenGameButton();
-  void displayNextPlayerButton();
   void displayRecyclingButtons();
-  void displayNormalGameButtons();
-  void displayDefenseButtons();
-  void displayInvasionButtons();
-  void displayCancelButton();
+  void clearHighlighting();
+  void startLocalCurrentAI();
   void displayDefenseWindow();
+  
   //@}
 
-  void clearGameActionsToolbar(bool send = true);
-
   /**
     * Updates the flag in the statusbar with the one of the current player
     */
@@ -402,7 +409,7 @@ public:
   void cancelShiftSource();
 
   /** Called when the user clicks the new game button. */
-  bool actionNewGame();
+  bool actionNewGame(GameLogic::GameAutomaton::NetworkGameType socket);
 
   /** Called when the user clicks the open game button. */
   bool actionOpenGame();
@@ -505,8 +512,8 @@ public:
     * long time ago and should probably be replaced by available armies local 
     * to players. 
     */
-  inline void availArmies(unsigned int nb) {m_nbAvailArmies = nb;}
-  inline unsigned int availArmies() {return m_nbAvailArmies;}
+/*  inline void availArmies(unsigned int nb) {m_nbAvailArmies = nb;}
+  inline unsigned int availArmies() {return m_nbAvailArmies;}*/
   //@}
 
   //@{
@@ -531,7 +538,7 @@ public:
     * Returns the list of players definitions whose connection from the network
     * is waited after loading a saved game. 
     */
-  inline std::vector<GameLogic::PlayerMatrix>& waitedPlayers() {return m_waitedPlayers;}
+  inline QList<GameLogic::PlayerMatrix>& waitedPlayers() {return m_waitedPlayers;}
 
   /** Displays the buttons associated to the given game state. */
   void displayButtonsForState(GameLogic::GameAutomaton::GameState state);
@@ -543,7 +550,7 @@ public:
   void winner(const GameLogic::Player* player);
 
   /** Returns a list of the nations names associated to their flag's file name. */
-  std::map< QString, QString > nationsList();
+  QMap< QString, QString > nationsList();
   
   /** Returns a pointer to the chat widget used to chat and to display messages. */
   inline KGameChat* chatWidget() {return m_chatDlg;}
@@ -569,16 +576,16 @@ public:
   void showMainMenu();
 
   /**
-    * The three types of central widget possible. This enum is used for knowing witch
-    * is currently displayed (see m_currentDisplayedWidget var below).
+    * The three types of possible central widget. This enum is used to know which
+    * one is currently displayed (see m_currentDisplayedWidget member below).
     */
-  enum widgetType {mainMenuType, mapType,arenaType};
+  enum WidgetType {MainMenu, Map, Arena};
 
   /**
     * Give type of the central widget currently displayed.
     * @return current widget type
     */
-  widgetType currentWidgetType();
+  WidgetType currentWidgetType();
 
   /**
     * Give the central widget currently displayed.
@@ -602,12 +609,41 @@ public:
     */
   GameLogic::Player* currentPlayer();
 
-  void slideInvade(GameLogic::Country *,GameLogic::Country *, InvasionType invasionType = Invasion);
+  void slideInvade(GameLogic::Country *,GameLogic::Country *, Ksirk::InvasionSlider::InvasionType invasionType = Ksirk::InvasionSlider::Invasion);
 
   void setNextPlayerActionEnabled(bool value);
   void setSaveGameActionEnabled(bool value);
 
-protected:
+  void updateScrollArrows();
+
+  bool newGameDialog(const QString& skin, bool networkGame);
+
+  bool finishSetupPlayers();
+
+  inline JabberClient* jabberClient() {return m_jabberClient;}
+  void askForJabberGames();
+  void sendGameInfoToJabber();
+
+  inline XMPP::Jid& serverJid() {return m_serverJid;}
+  inline void setServerJid(const XMPP::Jid& jid) {m_serverJid = jid;}
+
+  /**
+  * Sets our own presence. Updates our resource in the
+  * resource pool and sends a presence packet to the server.
+  */
+  void setPresence ( const XMPP::Status &status );
+
+  inline void setGroupchatHost(const QString& str) {m_groupchatHost = str;}
+  inline void setGroupchatRoom(const QString& str) {m_groupchatRoom = str;}
+  inline void setGroupchatNick(const QString& str) {m_groupchatNick = str;}
+  inline void setGroupchatPassword(const QString& str) {m_groupchatPassword = str;}
+  
+  const QString& groupchatHost() const {return m_groupchatHost;}
+  const QString& groupchatRoom() const {return m_groupchatRoom;}
+  const QString& groupchatNick() const {return m_groupchatNick;}
+  const QString& groupchatPassword() const {return m_groupchatPassword;}
+  
+  protected:
 
   /**
     * Connected to the frame timer, it manages the behavior of the game in
@@ -634,16 +670,6 @@ protected:
     */
   void initView();
 
-  /**
-    * Add a button to the toolbar
-    */
-  void addAButton(const QString& fileName, 
-      const char* slot, 
-      const QString& txt, 
-      const  KShortcut& shortcut, 
-      bool isTemp = false, 
-      const std::string& toolBarName = "gameActionsToolBar");
-
   /**
     * Reimplementation of the inherited function : starts the timer.
     */
@@ -668,28 +694,21 @@ protected:
   void reduceChat();
   void unreduceChat();
 
+Q_SIGNALS:
+    void newJabberGame(const QString&, int, const QString&);
+    
 public Q_SLOTS:
 
   virtual void mouseMoveEvent ( QMouseEvent * event );
 
-  void slideMove(int v);
-  void slideReleased();  
-  void slideClose();
-  //@{
-  /**
-   * The slots associated to the keys
-   * Will initiate different actions depending on the game state
-   */
-  void slotKey1();
-  void slotKey2();
-  void slotKey3();
-  //@}
-  
   //@{
   /**
     * The slots associated to the buttons
     */
+  void slotJabberGame();
   void slotNewGame();
+  void slotNewJabberGame();
+  void slotNewSocketGame();
   void slotJoinNetworkGame();
   void slotOpenGame();
   void slotSaveGame();
@@ -761,6 +780,12 @@ public Q_SLOTS:
 
   void slotZoomIn();
   void slotZoomOut();
+
+  void slotNewGameOK(unsigned int nbPlayers, const QString& skin, unsigned int nbNetworkPlayers, bool useGoals);
+  void slotNewGameKO();
+
+  void slotJabberGameCanceled(int previousIndex);
+  
 private Q_SLOTS:
   void optionsConfigure();
 
@@ -778,9 +803,98 @@ private Q_SLOTS:
   void slotDisableHelp(const QString &);
 
   void slotArmiesNumberChanged(int);
-  
-private:
 
+  /* Connects to the server. */
+//   void slotConnect ();
+  
+  /* Disconnects from the server. */
+//   void slotDisconnect ();
+  
+  // handle a TLS warning
+  void slotHandleTLSWarning ( QCA::TLS::IdentityResult identityResult, QCA::Validity validityResult );
+  
+  // handle client errors
+  void slotClientError ( JabberClient::ErrorCode errorCode );
+  
+  // we are connected to the server
+  void slotConnected ();
+  
+  /* Called from Psi: tells us when we've been disconnected from the server. */
+  void slotCSDisconnected ();
+  
+  /* Called from Psi: alerts us to a protocol error. */
+  void slotCSError (int);
+  
+  /* Called from Psi: roster request finished */
+  void slotRosterRequestFinished ( bool success );
+  
+  /* Called from Psi: incoming file transfer */
+//   void slotIncomingFileTransfer ();
+  
+  /* Called from Psi: debug messages from the backend. */
+  void slotClientDebugMessage (const QString &msg);
+  
+  /* XMPP console dialog */
+//   void slotXMPPConsole ();
+  
+  /* Slots for handling groupchats. */
+//   void slotJoinNewChat ();
+  void slotGroupChatJoined ( const XMPP::Jid &jid );
+  void slotGroupChatLeft ( const XMPP::Jid &jid );
+  void slotGroupChatPresence ( const XMPP::Jid &jid, const XMPP::Status &status );
+  void slotGroupChatError ( const XMPP::Jid &jid, int error, const QString &reason );
+  
+  /* Incoming subscription request. */
+//   void slotSubscription ( const XMPP::Jid &jid, const QString &type );
+  
+  /* the dialog that asked to add the contact was closed   (that dialog is shown in slotSubscription) */
+//   void slotAddedInfoEventActionActivated ( uint actionId );
+  
+  /**
+  * A new item appeared in our roster, synch it with the
+  * contact list.
+  * (or the contact has been updated
+  */
+//   void slotContactUpdated ( const XMPP::RosterItem & );
+  
+  /**
+  * An item has been deleted from our roster,
+  * delete it from our contact pool.
+  */
+//   void slotContactDeleted ( const XMPP::RosterItem & );
+  
+  
+  /* Someone on our contact list had (another) resource come online. */
+//   void slotResourceAvailable ( const XMPP::Jid &, const XMPP::Resource & );
+  
+  /* Someone on our contact list had (another) resource go offline. */
+//   void slotResourceUnavailable ( const XMPP::Jid &, const XMPP::Resource & );
+  
+  /* Displays a new message. */
+  void slotReceivedMessage ( const XMPP::Message & );
+  
+  /* Gets the user's vCard from the server for editing. */
+//   void slotEditVCard ();
+  
+  /* Get the services list from the server for management. */
+//   void slotGetServices ();
+  
+  /* we received a voice invitation */
+//  void slotIncomingVoiceCall(const Jid&);
+
+/* the unregister task finished */
+//   void slotUnregisterFinished();
+  void slotExit();
+  
+private: // Private methods
+  void createDefenseDialog();
+  void moveArmies(GameLogic::Country& src, GameLogic::Country& dest, unsigned int nb);
+  void saveXml(std::ostream& xmlStream);
+  void loadDices();
+  QPixmap buildDice(DiceColor color, const QString& id);
+  void setupPopupMessage();
+    
+private: // Private members
   QDockWidget * m_rightDock;
 
   KRightDialog * m_rightDialog;
@@ -789,21 +903,12 @@ private:
 
   GameLogic::GameAutomaton* m_automaton;
 
-  KDialog * m_wSlide;
-
-  int m_nbRArmy;
-  int m_nbLArmy;
-  int m_currentSlideValue;
-  int m_previousSlideValue;
-
-  QLabel * m_nbLArmies;
-  QLabel * m_nbRArmies;
-  QSlider * m_invadeSlide;
+  InvasionSlider * m_wSlide;
 
   /**
     * State that say the widget that is currently displayed between the map and the arena.
     */
-  widgetType m_currentDisplayedWidget;
+  WidgetType m_currentDisplayedWidget;
 
   /**
    * The widget initialy docked at bottom where is displayed the events history
@@ -821,7 +926,7 @@ private:
   int NKD, NKA;
   //@}
   
-  bool ARENA;
+  bool m_useArena;
 
   int nbSpriteAttacking;
   int nbSpriteDefending;
@@ -903,21 +1008,22 @@ private:
     * This button is used to display the flag of the currently active player
     * in the status bar.
     */
-  QAction* m_goalAction;
+  KAction* m_goalAction;
+  QAction* m_jabberAction;
   QLabel* m_barFlag;
     
 //   KAccel m_accels;
   
-  std::set< QString > m_temporaryAccelerators;
+  QList<QString> m_temporaryAccelerators;
     
   /** Used during countries distribution to handle network lags on the player member */
-  unsigned int m_nbAvailArmies; 
+//   unsigned int m_nbAvailArmies; 
   
   /** 
     * The list of players description whose connection is waited after loading 
     * a saved game. 
     */
-  std::vector<GameLogic::PlayerMatrix> m_waitedPlayers;
+  QList<GameLogic::PlayerMatrix> m_waitedPlayers;
 
   /**
    * The prompter where the game events and chat between network players are 
@@ -938,7 +1044,7 @@ private:
   /**
     * Contains all dices of the game.
     */
-  std::map< DiceColor, std::vector<QPixmap> > m_dices;
+  QMap< DiceColor, QList<QPixmap> > m_dices;
 
   /**
     * Audio player object: play all the sounds of the game.
@@ -953,17 +1059,13 @@ private:
 
   QList<AnimSpritesGroup*> m_animSpritesGroups;
 
-  KToolBar* gameActionsToolBar;
-
   KGamePopupItem * m_message;
 
   GameLogic::Country* m_mouseLocalisation;
 
-  KDialog * dial;
+  KDialog * m_defenseDialog;
 
   // components that will be re-used of the chat
-  QPixmap m_upChatReducePix;
-  QPixmap m_downChatReducePix;
   QPixmap m_upChatFloatPix;
   QPixmap m_downChatFloatPix;
 
@@ -977,15 +1079,39 @@ private:
   QString m_fileName;
 
   QAction* m_nextPlayerAction;
-  QAction* m_saveGameAction;
+  KAction* m_saveGameAction;
 
-private: // Private methods
-  /*void attack(GameLogic::Country& attacker, GameLogic::Country& defender, unsigned int nb);*/
-  void moveArmies(GameLogic::Country& src, GameLogic::Country& dest, unsigned int nb);
-  void saveXml(std::ostream& xmlStream);
-  void loadDices();
-  QPixmap buildDice(DiceColor color, const QString& id);
-  void setupPopupMessage();
+  Sprites::ArrowSprite* m_uparrow;
+  Sprites::ArrowSprite* m_downarrow;
+  Sprites::ArrowSprite* m_leftarrow;
+  Sprites::ArrowSprite* m_rightarrow;
+  
+  bool m_networkGame;
+  int m_port;
+  uint m_newPlayersNumber;
+  bool m_reinitializingGame;
+  
+  NewGameDialogImpl* m_newGameDialog;
+  
+  GameLogic::GameAutomaton::GameState m_stateBeforeNewGame;
+  int m_stackWidgetBeforeNewGame;
+  
+  JabberClient* m_jabberClient;
+  XMPP::Jid m_serverJid;
+  
+  /* Initial presence to set after connecting. */
+  XMPP::Status m_initialPresence;
+  
+  QString m_groupchatHost;
+  QString m_groupchatRoom;
+  QString m_groupchatNick;
+  QString m_groupchatPassword;
+  
+  QString m_advertizedHostName;
+  
+  KsirkJabberGameWidget* m_jabberGameWidget;
+  
+  QSet<QString> m_presents;
 
 };
 
