@@ -20,7 +20,7 @@
 static const KLocalizedString description = ki18n("Potato game for kids");
 static const KLocalizedString text = ki18n("A program original by <email address='%1'>Éric Bischoff</email>\nand John Calhoun.\n\nThis program is dedicated to my daughter Sunniva.").subs("ebischoff@nerim.net");
 
-static const char version[] = "0.5.9";
+static const char version[] = "0.6.0";
 
 // Main function
 int main(int argc, char *argv[])
@@ -28,10 +28,12 @@ int main(int argc, char *argv[])
 
   KAboutData aboutData( "ktuberling", 0, ki18n("KTuberling"), 
     version, description, KAboutData::License_GPL, 
+    // TODO KDE 4.4: Update copyright years
     ki18n("(c) 1999-2006, The KTuberling Developers"), text, "http://games.kde.org/ktuberling" );
-  aboutData.addAuthor(ki18n("Éric Bischoff"), ki18n("Developer"), "ebischoff@nerim.net");
-  aboutData.addAuthor(ki18n("John Calhoun"), ki18n("Original concept and artwork"));
+   // TODO KDE 4.4: If Albert still maintainer change "SVG port" -> "Maintainer"; "Developer" -> "Former Developer"
   aboutData.addAuthor(ki18n("Albert Astals Cid"), ki18n("SVG port"), "aacid@kde.org");
+  aboutData.addAuthor(ki18n("Éric Bischoff"), ki18n("Developer"), "ebischoff@nerim.net");
+  aboutData.addCredit(ki18n("John Calhoun"), ki18n("Original concept and artwork"));
   aboutData.addCredit(ki18n("Agnieszka Czajkowska"), ki18n("New artwork"), "agnieszka@imagegalaxy.de");
   aboutData.addCredit(ki18n("Bas Willems"), ki18n("New artwork"), "cybersurfer@euronet.nl");
   aboutData.addCredit(ki18n("Roger Larsson"), ki18n("Sounds tuning"), "roger.larsson@norran.net");
