@@ -1,6 +1,6 @@
 /*
     This file is part of the KDE games library
-    Copyright (C) 2001 Martin Heni (martin@heni-online.de)
+    Copyright (C) 2001 Martin Heni (kde at heni-online.de)
     Copyright (C) 2001 Andreas Beckermann (b_mann@gmx.de)
 
     This library is free software; you can redistribute it and/or
@@ -17,9 +17,6 @@
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
 */
-/*
-    $Id: kgamenetwork.cpp 465369 2005-09-29 14:33:08Z mueller $
-*/
 
 #include "kgamenetwork.h"
 #include "kgamenetwork.moc"
@@ -34,6 +31,8 @@
 #include <kdebug.h>
 
 #include <qbuffer.h>
+//Added by qt3to4:
+#include <QList>
 
 
 class KGameNetworkPrivate
@@ -50,7 +49,7 @@ public:
 public:
         KMessageClient* mMessageClient;
         KMessageServer* mMessageServer;
-        Q_UINT32 mDisconnectId;  // Stores gameId() over a disconnect process
+        quint32 mDisconnectId;  // Stores gameId() over a disconnect process
 	DNSSD::PublicService* mService;
 	QString mType;
 	QString mName;
@@ -59,21 +58,22 @@ public:
 };
 
 // ------------------- NETWORK GAME ------------------------
-KGameNetwork::KGameNetwork(int c, QObject* parent) : QObject(parent, 0)
+KGameNetwork::KGameNetwork(int c, QObject* parent) 
+    : QObject(parent),
+      d( new KGameNetworkPrivate )
 {
- d = new KGameNetworkPrivate;
- d->mCookie = (Q_INT16)c;
+ d->mCookie = (qint16)c;
 
  // Init the game as a local game, i.e.
  // create your own KMessageServer and a KMessageClient connected to it.
  setMaster();
 
- kdDebug(11001) << k_funcinfo << "this=" << this <<", cookie=" << cookie() << " sizeof(this)="<<sizeof(KGameNetwork) << endl;
+ kDebug(11001) << "this=" << this <<", cookie=" << cookie() << "sizeof(this)="<<sizeof(KGameNetwork);
 }
 
 KGameNetwork::~KGameNetwork()
 {
- kdDebug(11001) << k_funcinfo << "this=" << this << endl;
+ kDebug(11001) << "this=" << this;
 // Debug();
  delete d->mService;
  delete d;
@@ -83,7 +83,7 @@ KGameNetwork::~KGameNetwork()
 bool KGameNetwork::isNetwork() const
 { return isOfferingConnections() || d->mMessageClient->isNetwork();}
 
-Q_UINT32 KGameNetwork::gameId() const
+quint32 KGameNetwork::gameId() const
 {
   //return d->mMessageClient->id() ;
   // Return stored id in the case of disconnect. In any other
@@ -116,40 +116,40 @@ void KGameNetwork::setMaster()
  if (!d->mMessageServer) {
    d->mMessageServer = new KMessageServer (cookie(), this);
  } else {
-   kdWarning(11001) << k_funcinfo << "Server already running!!" << endl;
+   kWarning(11001) << "Server already running!!";
  }
  if (!d->mMessageClient) {
    d->mMessageClient = new KMessageClient (this);
-   connect (d->mMessageClient, SIGNAL(broadcastReceived(const QByteArray&, Q_UINT32)),
-            this, SLOT(receiveNetworkTransmission(const QByteArray&, Q_UINT32)));
+   connect (d->mMessageClient, SIGNAL(broadcastReceived(const QByteArray&, quint32)),
+            this, SLOT(receiveNetworkTransmission(const QByteArray&, quint32)));
    connect (d->mMessageClient, SIGNAL(connectionBroken()),
             this, SIGNAL(signalConnectionBroken()));
-   connect (d->mMessageClient, SIGNAL(aboutToDisconnect(Q_UINT32)),
-            this, SLOT(aboutToLoseConnection(Q_UINT32)));
+   connect (d->mMessageClient, SIGNAL(aboutToDisconnect(quint32)),
+            this, SLOT(aboutToLoseConnection(quint32)));
    connect (d->mMessageClient, SIGNAL(connectionBroken()),
             this, SLOT(slotResetConnection()));
 
    connect (d->mMessageClient, SIGNAL(adminStatusChanged(bool)),
             this, SLOT(slotAdminStatusChanged(bool)));
-   connect (d->mMessageClient, SIGNAL(eventClientConnected(Q_UINT32)),
-            this, SIGNAL(signalClientConnected(Q_UINT32)));
-   connect (d->mMessageClient, SIGNAL(eventClientDisconnected(Q_UINT32, bool)),
-            this, SIGNAL(signalClientDisconnected(Q_UINT32, bool)));
+   connect (d->mMessageClient, SIGNAL(eventClientConnected(quint32)),
+            this, SIGNAL(signalClientConnected(quint32)));
+   connect (d->mMessageClient, SIGNAL(eventClientDisconnected(quint32, bool)),
+            this, SIGNAL(signalClientDisconnected(quint32, bool)));
 
    // broacast and direct messages are treated equally on receive.
-   connect (d->mMessageClient, SIGNAL(forwardReceived(const QByteArray&, Q_UINT32, const QValueList<Q_UINT32>&)),
-            d->mMessageClient, SIGNAL(broadcastReceived(const QByteArray&, Q_UINT32)));
+   connect (d->mMessageClient, SIGNAL(forwardReceived(const QByteArray&, quint32, const QList<quint32>&)),
+            d->mMessageClient, SIGNAL(broadcastReceived(const QByteArray&, quint32)));
 
  } else {
    // should be no problem but still has to be tested
-   kdDebug(11001) << k_funcinfo << "Client already exists!" << endl;
+   kDebug(11001) << "Client already exists!";
  }
  d->mMessageClient->setServer(d->mMessageServer);
 }
 
 void KGameNetwork::setDiscoveryInfo(const QString& type, const QString& name)
 {
- kdDebug() << k_funcinfo << type << ":" << name << endl;
+ kDebug() << type << ":" << name;
  d->mType = type;
  d->mName = name;
  tryPublish();
@@ -171,9 +171,9 @@ void KGameNetwork::tryStopPublishing()
  if (d->mService) d->mService->stop();
 }
 
-bool KGameNetwork::offerConnections(Q_UINT16 port)
+bool KGameNetwork::offerConnections(quint16 port)
 {
- kdDebug (11001) << k_funcinfo << "on port " << port << endl;
+ kDebug (11001) << "on port" << port;
  if (!isMaster()) {
    setMaster();
  }
@@ -183,63 +183,75 @@ bool KGameNetwork::offerConnections(Q_UINT16 port)
 
  // FIXME: This debug message can be removed when the program is working correct.
  if (d->mMessageServer && d->mMessageServer->isOfferingConnections()) {
-   kdDebug (11001) << k_funcinfo << "Already running as server! Changing the port now!" << endl;
+   kDebug (11001) << "Already running as server! Changing the port now!";
  }
 
  tryStopPublishing();
- kdDebug (11001) << k_funcinfo << "before Server->initNetwork" << endl;
+ kDebug (11001) << "before Server->initNetwork";
  if (!d->mMessageServer->initNetwork (port)) {
-   kdError (11001) << k_funcinfo << "Unable to bind to port " << port << "!" << endl;
+   kError (11001) << "Unable to bind to port" << port << "!";
    // no need to delete - we just cannot listen to the port
 //   delete d->mMessageServer;
 //   d->mMessageServer = 0;
 //   d->mMessageClient->setServer((KMessageServer*)0);
    return false;
  }
- kdDebug (11001) << k_funcinfo << "after Server->initNetwork" << endl;
+ kDebug (11001) << "after Server->initNetwork";
  tryPublish();
  return true;
 }
 
-bool KGameNetwork::connectToServer (const QString& host, Q_UINT16 port)
+bool KGameNetwork::connectToServer (const QString& host, quint16 port)
 {
  if (host.isEmpty()) {
-   kdError(11001) << k_funcinfo << "No hostname given" << endl;
+   kError(11001) << "No hostname given";
    return false;
  }
+ if (connectToServer(new KMessageSocket (host, port)))
+ {
+  kDebug(11001) << "connected to" << host << ":" << port;
+  return true;
+ }
+ else
+ {
+   return false;
+ }
+}
 
- // Make sure this is 0
- d->mDisconnectId = 0;
-
-// if (!d->mMessageServer) {
-//   // FIXME: What shall we do here? Probably must stop a running game.
-//   kdWarning (11001) << k_funcinfo << "We are already connected to another server!" << endl;
-/// }
-
+bool KGameNetwork::connectToServer (KMessageIO *connection)
+{
+  // Make sure this is 0
+  d->mDisconnectId = 0;
+  
+  // if (!d->mMessageServer) {
+ //   // FIXME: What shall we do here? Probably must stop a running game.
+ //   kWarning (11001) << "We are already connected to another server!";
+ /// }
+ 
  if (d->mMessageServer) {
    // FIXME: What shall we do here? Probably must stop a running game.
-   kdWarning(11001) << "we are server but we are trying to connect to another server! "
-                    << "make sure that all clients connect to that server! "
-                    << "quitting the local server now..." << endl;
+   kWarning(11001) << "we are server but we are trying to connect to another server! "
+   << "make sure that all clients connect to that server! "
+   << "quitting the local server now...";
    stopServerConnection();
    d->mMessageClient->setServer((KMessageIO*)0);
    delete d->mMessageServer;
    d->mMessageServer = 0;
  }
-
- kdDebug(11001) << "    about to set server" << endl;
- d->mMessageClient->setServer(host, port);
+ 
+ kDebug(11001) << "    about to set server";
+ d->mMessageClient->setServer(connection);
  emit signalAdminStatusChanged(false); // as we delete the connection above isAdmin() is always false now!
-
+ 
  // OK: We say that we already have connected, but this isn't so yet!
  // If the connection cannot be established, it will look as being disconnected
  // again ("slotConnectionLost" is called).
  // Shall we differ between these?
- kdDebug(11001) << "connected to " << host << ":" << port << endl;
+ kDebug(11001) << "connected";
  return true;
 }
 
-Q_UINT16 KGameNetwork::port() const
+quint16 KGameNetwork::port() const
 {
  if (isNetwork()) {
    if (isOfferingConnections()) {
@@ -273,23 +285,23 @@ bool KGameNetwork::isOfferingConnections() const
 void KGameNetwork::disconnect()
 {
  // TODO MH
- kdDebug(11001) << k_funcinfo << endl;
+ kDebug(11001) ;
  stopServerConnection();
  if (d->mMessageServer) {
-    QValueList <Q_UINT32> list=d->mMessageServer->clientIDs();
-    QValueList<Q_UINT32>::Iterator it;
+    QList <quint32> list=d->mMessageServer->clientIDs();
+    QList<quint32>::Iterator it;
     for( it = list.begin(); it != list.end(); ++it )
     {
-      kdDebug(11001) << "Client id=" << (*it) <<  endl;
+      kDebug(11001) << "Client id=" << (*it);
       KMessageIO *client=d->mMessageServer->findClient(*it);
       if (!client)
       {
         continue;
       }
-      kdDebug(11001) << "   rtti=" << client->rtti() <<  endl;
+      kDebug(11001) << "   rtti=" << client->rtti();
       if (client->rtti()==2)
       {
-        kdDebug(11001) << "DIRECT IO " << endl;
+        kDebug(11001) << "DIRECT IO";
       }
       else
       {
@@ -299,12 +311,12 @@ void KGameNetwork::disconnect()
  }
  else
  {
-   kdDebug(11001) << k_funcinfo << "before client->disconnect() id="<<gameId()<< endl;
+   kDebug(11001) << "before client->disconnect() id="<<gameId();
    //d->mMessageClient->setServer((KMessageIO*)0);
-   kdDebug(11001) << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++"<<endl;
+   kDebug(11001) << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++";
    d->mMessageClient->disconnect();
 
-   kdDebug(11001) << "++++++--------------------------------------------+++++"<<endl;
+   kDebug(11001) << "++++++--------------------------------------------+++++";
  }
  //setMaster();
  /*
@@ -312,35 +324,35 @@ void KGameNetwork::disconnect()
   //delete d->mMessageServer;
   //d->mMessageServer=0;
   server=true;
-  kdDebug(11001) << "  server true" << endl;
+  kDebug(11001) << "  server true";
   d->mMessageServer->deleteClients();
-  kdDebug(11001) << "  server deleteClients" << endl;
+  kDebug(11001) << "  server deleteClients";
  }
  */
- kdDebug(11001) << k_funcinfo << "DONE" << endl;
+ kDebug(11001) << "DONE";
 }
 
-void KGameNetwork::aboutToLoseConnection(Q_UINT32 clientID)
+void KGameNetwork::aboutToLoseConnection(quint32 clientID)
 {
-  kdDebug(11001) << "Storing client id of connection "<<clientID<<endl;
+  kDebug(11001) << "Storing client id of connection "<<clientID;
   d->mDisconnectId = clientID;
 }
 
 void KGameNetwork::slotResetConnection()
 {
-  kdDebug(11001) << "Resseting client disconnect id"<<endl;
+  kDebug(11001) << "Resseting client disconnect id";
   d->mDisconnectId = 0;
 }
 
-void KGameNetwork::electAdmin(Q_UINT32 clientID)
+void KGameNetwork::electAdmin(quint32 clientID)
 {
  if (!isAdmin()) {
-	kdWarning(11001) << k_funcinfo << "only ADMIN is allowed to call this!" << endl;
+	kWarning(11001) << "only ADMIN is allowed to call this!";
 	return;
  }
  QByteArray buffer;
- QDataStream stream(buffer,IO_WriteOnly);
- stream << static_cast<Q_UINT32>( KMessageServer::REQ_ADMIN_CHANGE );
+ QDataStream stream(&buffer,QIODevice::WriteOnly);
+ stream << static_cast<quint32>( KMessageServer::REQ_ADMIN_CHANGE );
  stream << clientID;
  d->mMessageClient->sendServerMessage(buffer);
 }
@@ -348,13 +360,13 @@ void KGameNetwork::electAdmin(Q_UINT32 clientID)
 void KGameNetwork::setMaxClients(int max)
 {
  if (!isAdmin()) {
-	kdWarning(11001) << k_funcinfo << "only ADMIN is allowed to call this!" << endl;
+	kWarning(11001) << "only ADMIN is allowed to call this!";
 	return;
  }
  QByteArray buffer;
- QDataStream stream(buffer,IO_WriteOnly);
- stream << static_cast<Q_UINT32>( KMessageServer::REQ_MAX_NUM_CLIENTS );
- stream << (Q_INT32)max;
+ QDataStream stream(&buffer,QIODevice::WriteOnly);
+ stream << static_cast<quint32>( KMessageServer::REQ_MAX_NUM_CLIENTS );
+ stream << (qint32)max;
  d->mMessageClient->sendServerMessage(buffer);
 }
 
@@ -374,50 +386,50 @@ void KGameNetwork::unlock()
 
 // --------------------- send messages ---------------------------
 
-bool KGameNetwork::sendSystemMessage(int data, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendSystemMessage(int data, int msgid, quint32 receiver, quint32 sender)
 {
  QByteArray buffer;
- QDataStream stream(buffer,IO_WriteOnly);
+ QDataStream stream(&buffer,QIODevice::WriteOnly);
  stream << data;
  return sendSystemMessage(buffer,msgid,receiver,sender);
 }
 
-bool KGameNetwork::sendSystemMessage(const QString &msg, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendSystemMessage(const QString &msg, int msgid, quint32 receiver, quint32 sender)
 {
  QByteArray buffer;
- QDataStream stream(buffer, IO_WriteOnly);
+ QDataStream stream(&buffer, QIODevice::WriteOnly);
  stream << msg;
  return sendSystemMessage(buffer, msgid, receiver, sender);
 }
 
-bool KGameNetwork::sendSystemMessage(const QDataStream &msg, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendSystemMessage(const QDataStream &msg, int msgid, quint32 receiver, quint32 sender)
 { return sendSystemMessage(((QBuffer*)msg.device())->buffer(), msgid, receiver, sender); }
 
-bool KGameNetwork::sendSystemMessage(const QByteArray& data, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendSystemMessage(const QByteArray& data, int msgid, quint32 receiver, quint32 sender)
 {
  QByteArray buffer;
- QDataStream stream(buffer,IO_WriteOnly);
+ QDataStream stream(&buffer,QIODevice::WriteOnly);
  if (!sender) {
    sender = gameId();
  }
 
- Q_UINT32 receiverClient = KGameMessage::rawGameId(receiver); // KGame::gameId()
+ quint32 receiverClient = KGameMessage::rawGameId(receiver); // KGame::gameId()
  int receiverPlayer = KGameMessage::rawPlayerId(receiver); // KPlayer::id()
 
  KGameMessage::createHeader(stream, sender, receiver, msgid);
- stream.writeRawBytes(data.data(), data.size());
+ stream.writeRawData(data.data(), data.size());
 
  /*
- kdDebug(11001) << "transmitGameClientMessage msgid=" << msgid << " recv="
-                << receiver << " sender=" << sender << " Buffersize="
-                << buffer.size() << endl;
+ kDebug(11001) << "transmitGameClientMessage msgid=" << msgid << "recv="
+                << receiver << "sender=" << sender << "Buffersize="
+                << buffer.size();
   */
 
  if (!d->mMessageClient) {
    // No client created, this should never happen!
    // Having a local game means we have our own
    // KMessageServer and we are the only client.
-   kdWarning (11001) << k_funcinfo << "We don't have a client! Should never happen!" << endl;
+   kWarning (11001) << "We don't have a client! Should never happen!";
    return false;
  }
 
@@ -435,37 +447,37 @@ bool KGameNetwork::sendSystemMessage(const QByteArray& data, int msgid, Q_UINT32
  return true;
 }
 
-bool KGameNetwork::sendMessage(int data, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendMessage(int data, int msgid, quint32 receiver, quint32 sender)
 { return sendSystemMessage(data,msgid+KGameMessage::IdUser,receiver,sender); }
 
-bool KGameNetwork::sendMessage(const QString &msg, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendMessage(const QString &msg, int msgid, quint32 receiver, quint32 sender)
 { return sendSystemMessage(msg,msgid+KGameMessage::IdUser,receiver,sender); }
 
-bool KGameNetwork::sendMessage(const QDataStream &msg, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendMessage(const QDataStream &msg, int msgid, quint32 receiver, quint32 sender)
 { return sendSystemMessage(msg, msgid+KGameMessage::IdUser, receiver, sender); }
 
-bool KGameNetwork::sendMessage(const QByteArray &msg, int msgid, Q_UINT32 receiver, Q_UINT32 sender)
+bool KGameNetwork::sendMessage(const QByteArray &msg, int msgid, quint32 receiver, quint32 sender)
 { return sendSystemMessage(msg, msgid+KGameMessage::IdUser, receiver, sender); }
 
-void KGameNetwork::sendError(int error,const QByteArray& message, Q_UINT32 receiver, Q_UINT32 sender)
+void KGameNetwork::sendError(int error,const QByteArray& message, quint32 receiver, quint32 sender)
 {
  QByteArray buffer;
- QDataStream stream(buffer,IO_WriteOnly);
- stream << (Q_INT32) error;
- stream.writeRawBytes(message.data(), message.size());
+ QDataStream stream(&buffer,QIODevice::WriteOnly);
+ stream << (qint32) error;
+ stream.writeRawData(message.data(), message.size());
  sendSystemMessage(stream,KGameMessage::IdError,receiver,sender);
 }
 
 
 // ----------------- receive messages from the network
-void KGameNetwork::receiveNetworkTransmission(const QByteArray& receiveBuffer, Q_UINT32 clientID)
+void KGameNetwork::receiveNetworkTransmission(const QByteArray& receiveBuffer, quint32 clientID)
 {
- QDataStream stream(receiveBuffer, IO_ReadOnly);
+ QDataStream stream(receiveBuffer);
  int msgid;
- Q_UINT32 sender; // the id of the KGame/KPlayer who sent the message
- Q_UINT32 receiver; // the id of the KGame/KPlayer the message is for 
+ quint32 sender; // the id of the KGame/KPlayer who sent the message
+ quint32 receiver; // the id of the KGame/KPlayer the message is for 
  KGameMessage::extractHeader(stream, sender, receiver, msgid);
-// kdDebug(11001) << k_funcinfo << "id=" << msgid << " sender=" << sender << " recv=" << receiver << endl;
+// kDebug(11001) << "id=" << msgid << "sender=" << sender << "recv=" << receiver;
 
  // No broadcast : receiver==0
  // No player isPlayer(receiver)
@@ -473,19 +485,19 @@ void KGameNetwork::receiveNetworkTransmission(const QByteArray& receiveBuffer, Q
  if (receiver &&  receiver!=gameId() && !KGameMessage::isPlayer(receiver) )
  {
    // receiver=0 is broadcast or player message
-   kdDebug(11001) << k_funcinfo << "Message not meant for us "
-            << gameId() << "!=" << receiver << " rawid="
-            << KGameMessage::rawGameId(receiver) << endl;
+   kDebug(11001) << "Message not meant for us "
+            << gameId() << "!=" << receiver << "rawid="
+            << KGameMessage::rawGameId(receiver);
    return;
  }
  else if (msgid==KGameMessage::IdError)
  {
    QString text;
-   Q_INT32 error;
+   qint32 error;
    stream >> error;
-   kdDebug(11001) << k_funcinfo << "Got IdError " << error << endl;
+   kDebug(11001) << "Got IdError" << error;
    text = KGameError::errorText(error, stream);
-   kdDebug(11001) << "Error text: " << text.latin1() << endl;
+   kDebug(11001) << "Error text:" << text.toLatin1();
    emit signalNetworkErrorMessage((int)error,text);
  }
  else
@@ -504,11 +516,11 @@ void KGameNetwork::slotAdminStatusChanged(bool isAdmin)
 
 void KGameNetwork::Debug()
 {
- kdDebug(11001) << "------------------- KNETWORKGAME -------------------------" << endl;
- kdDebug(11001) << "gameId         " << gameId() << endl;
- kdDebug(11001) << "gameMaster     " << isMaster() << endl;
- kdDebug(11001) << "gameAdmin      " << isAdmin() << endl;
- kdDebug(11001) << "---------------------------------------------------" << endl;
+ kDebug(11001) << "------------------- KNETWORKGAME -------------------------";
+ kDebug(11001) << "gameId         " << gameId();
+ kDebug(11001) << "gameMaster     " << isMaster();
+ kDebug(11001) << "gameAdmin      " << isAdmin();
+ kDebug(11001) << "---------------------------------------------------";
 }
 
 /*
